/********************* SniipSoftel *****************
 *
 *   $Author: Autobuild $
 *   $Revision: 2 $
 *   $Date: 19.12.06 23:18 $
 *
 ********************* SniipSoftel *****************/
using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using Softel.Windows.Forms.CuteGridColumns;

namespace Softel.Windows.Forms
{
	/// <summary>
	/// Summary description for CuteGridDateTimeColumn.
	/// </summary>
	public class CuteGridComboBoxColumn : DataGridColumnStyle
	{
		private FlatComboBox m_cmbEdit = new FlatComboBox();
		// The m_bIsEditing field tracks whether or not the user is
		// editing data with the hosted control.
		private bool m_bIsEditing;
		private string m_strFormat;
		private IFormatProvider m_fpFormat;
		private HorizontalAlignment m_haValueAlignment;
		private EventHandler m_evtValueChanged;
		private DataTable m_dtData;
		private string m_strDataKey;
		private string m_strDataValue;

		public CuteGridComboBoxColumn() : base()
		{
			m_cmbEdit.Visible = false;
			m_cmbEdit.DropDownStyle = ComboBoxStyle.DropDownList;
			m_evtValueChanged = new EventHandler(TimePickerValueChanged);
		}

		public DataTable DataSource
		{
			get{ return m_dtData; }
			set{ m_dtData = value;}
		}
		public string DataValue
		{
			get { return m_strDataValue; }
			set { m_strDataValue = value; }
		}
		public string DataKey
		{
			get { return m_strDataKey; }
			set { m_strDataKey = value; }
		}
			public HorizontalAlignment ValueAlignment
		{
			get { return m_haValueAlignment; }
			set { m_haValueAlignment = value; }
		}

		public string Format
		{
			get { return m_strFormat; }
			set { m_strFormat = value; }
		}

		public IFormatProvider FormatInfo
		{
			get { return m_fpFormat; }
			set { m_fpFormat = value; }
		}

		protected override void Abort(int rowNum)
		{
			m_bIsEditing = false;
			m_cmbEdit.SelectedValueChanged -= m_evtValueChanged;
			Invalidate();
		}

		protected override bool Commit
			(CurrencyManager dataSource, int rowNum)
		{
			m_cmbEdit.Bounds = Rectangle.Empty;

			m_cmbEdit.SelectedValueChanged -= m_evtValueChanged;

			if (!m_bIsEditing)
				return true;

			m_bIsEditing = false;

			try
			{
				String strValue = m_cmbEdit.Text;
				SetColumnValueAtRow(dataSource, rowNum, strValue);
				((CuteSoftelGrid) this.DataGridTableStyle.DataGrid).CurrentRow[DataKey] = m_cmbEdit.SelectedValue;
//				SetColumnValueAtRow(dataSource, rowNum, strValue);
			}
			catch (Exception)
			{
				Abort(rowNum);
				return false;
			}

			Invalidate();
			return true;
		}

		protected override void Edit(
			CurrencyManager source,
			int rowNum,
			Rectangle bounds,
			bool readOnly,
			string instantText,
			bool cellIsVisible)
		{
//			string strValue = GetColumnValueAtRow(source, rowNum).ToString();
			object objValue = ((CuteSoftelGrid) this.DataGridTableStyle.DataGrid).CurrentRow[DataKey];
			if (cellIsVisible)
			{
				m_cmbEdit.Bounds = new Rectangle
					(bounds.X, bounds.Y,
					 bounds.Width, bounds.Height);
				m_cmbEdit.DataSource = DataSource;
				m_cmbEdit.DisplayMember = DataValue;
				m_cmbEdit.ValueMember = DataKey;
				m_cmbEdit.SelectedValue = objValue;
				m_cmbEdit.Visible = true;
				m_cmbEdit.SelectedValueChanged += m_evtValueChanged;
			}
			else
			{
				m_cmbEdit.SelectedValue = objValue;
				m_cmbEdit.Visible = false;
			}

			if (m_cmbEdit.Visible)
				DataGridTableStyle.DataGrid.Invalidate(bounds);
		}

		protected override Size GetPreferredSize(
			Graphics g,
			object value)
		{
			return new Size(100, m_cmbEdit.PreferredHeight);
		}

		protected override int GetMinimumHeight()
		{
			return m_cmbEdit.PreferredHeight;
		}

		protected override int GetPreferredHeight(Graphics g,
		                                          object value)
		{
			return m_cmbEdit.PreferredHeight;
		}

		protected override void Paint(Graphics g,
		                              Rectangle bounds,
		                              CurrencyManager source,
		                              int rowNum)
		{
			Paint(g, bounds, source, rowNum, false);
		}

		protected override void Paint(
			Graphics g,
			Rectangle bounds,
			CurrencyManager source,
			int rowNum,
			bool alignToRight)
		{
			Paint(
				g, bounds,
				source,
				rowNum,
				Brushes.Red,
				Brushes.Blue,
				alignToRight);
		}

		protected override void Paint(
			Graphics g,
			Rectangle bounds,
			CurrencyManager source,
			int rowNum,
			Brush backBrush,
			Brush foreBrush,
			bool alignToRight)
		{
			try
			{
				CuteGridPaintCellEventArgs e
					= new CuteGridPaintCellEventArgs(g, bounds, source, this, rowNum, GetColumnValueAtRow(source, rowNum),
					                                 backBrush, foreBrush, alignToRight);

				((CuteSoftelGrid) this.DataGridTableStyle.DataGrid).OnCellPaint(e);

				if (!e.CustomPaint)
				{
					if (this.DataGridTableStyle.DataGrid.IsSelected(rowNum))
					{
						backBrush = e.SelectionBackBrush;
						foreBrush = e.SelectionForeBrush;
					}
					else
					{
						backBrush = e.BackBrush;
						foreBrush = e.ForeBrush;
					}
//					DateTime date = (DateTime) 
//						GetColumnValueAtRow(source, rowNum);
					Rectangle rect = bounds;
					g.FillRectangle(backBrush, rect);
					rect.Offset(0, 2);
//					string strValue = date.ToString("d");
					string strValue = string.Empty;
					string strFormat = "{0:" + m_strFormat + "}";
					if (m_fpFormat == null)
						strValue = string.Format(strFormat, GetColumnValueAtRow(source, rowNum));
					else
						strValue = string.Format(m_fpFormat, strFormat, GetColumnValueAtRow(source, rowNum));
					int iWidth = g.MeasureString(strValue, this.DataGridTableStyle.DataGrid.Font).ToSize().Width;
					switch (m_haValueAlignment)
					{
						case HorizontalAlignment.Left:
							break;
						case HorizontalAlignment.Right:
							rect.X = rect.X + (rect.Width - iWidth);
							break;
						case HorizontalAlignment.Center:
							rect.X = rect.X + (rect.Width - iWidth)/2;
							break;
					}
					g.DrawString(strValue,
					             this.DataGridTableStyle.DataGrid.Font,
					             foreBrush, rect);
				}
			}
			catch (Exception)
			{
				base.Paint(g, bounds, source, rowNum, backBrush, foreBrush, alignToRight);
			}

		}

		protected override void SetDataGridInColumn(DataGrid value)
		{
			base.SetDataGridInColumn(value);
			if (m_cmbEdit.Parent != null)
			{
				m_cmbEdit.Parent.Controls.Remove
					(m_cmbEdit);
			}
			if (value != null)
			{
				value.Controls.Add(m_cmbEdit);
			}
		}

		private void TimePickerValueChanged(object sender, EventArgs e)
		{
			this.m_bIsEditing = true;
			base.ColumnStartedEditing(m_cmbEdit);
		}
	}
}
/********************* SniipSoftel *****************
 *
 *    $History: CuteGridComboBoxColumn.cs $
 *   
 *   *****************  Version 2  *****************
 *   User: Autobuild    Date: 19.12.06   Time: 23:18
 *   Updated in $/Net/Common/SoftelControl/CuteSoftelGrid/CuteGridColumns
 *
 ********************* SniipSoftel *****************/
