/********************* SniipSoftel *****************
 *
 *   $Author: Serg_ch $
 *   $Revision: 9 $
 *   $Date: 1.08.07 21:02 $
 *
 ********************* SniipSoftel *****************/
using System;
using System.Drawing;
using System.Windows.Forms;
using Softel.Windows.Forms.CuteGridColumns;

namespace Softel.Windows.Forms
{
	/// <summary>
	/// Summary description for CuteGridDateTimeColumn.
	/// </summary>
	public class CuteGridDateTimeColumn : DataGridColumnStyle
	{
		private DateTimePicker myDateTimePicker = new DateTimePicker();
		// The isEditing field tracks whether or not the user is
		// editing data with the hosted control.
		private bool isEditing;
		private string m_strFormat;
		private IFormatProvider m_fpFormat;
		private HorizontalAlignment m_haValueAlignment;

		public CuteGridDateTimeColumn() : base()
		{
			myDateTimePicker.Visible = false;
		}

		public HorizontalAlignment ValueAlignment
		{
			get { return m_haValueAlignment; }
			set { m_haValueAlignment = value; }
		}

		public string Format
		{
			get { return m_strFormat; }
			set { m_strFormat = value; }
		}

		public IFormatProvider FormatInfo
		{
			get { return m_fpFormat; }
			set { m_fpFormat = value; }
		}

		protected override void Abort(int rowNum)
		{
			isEditing = false;
			myDateTimePicker.ValueChanged -=
				new EventHandler(TimePickerValueChanged);
			Invalidate();
		}

		protected override bool Commit
			(CurrencyManager dataSource, int rowNum)
		{
			myDateTimePicker.Bounds = Rectangle.Empty;

			myDateTimePicker.ValueChanged -=
				new EventHandler(TimePickerValueChanged);

			if (!isEditing)
				return true;

			isEditing = false;

			try
			{
				DateTime value = myDateTimePicker.Value;
				SetColumnValueAtRow(dataSource, rowNum, value);
			}
			catch (Exception)
			{
				Abort(rowNum);
				return false;
			}

			Invalidate();
			return true;
		}

		protected override void Edit(
			CurrencyManager source,
			int rowNum,
			Rectangle bounds,
			bool readOnly,
			string instantText,
			bool cellIsVisible)
		{
			if(!readOnly)
			{
				object objValue = GetColumnValueAtRow(source, rowNum);
                if (objValue == DBNull.Value || !(objValue is DateTime))
					objValue = DateTime.Today;
				DateTime value = (DateTime)objValue;
				
				if (cellIsVisible)
				{
					myDateTimePicker.Bounds = new Rectangle
						(bounds.X, bounds.Y,
						 bounds.Width, bounds.Height);
					if(myDateTimePicker.MaxDate < value)
						myDateTimePicker.Value = myDateTimePicker.MaxDate;
					else
						myDateTimePicker.Value = value;
					myDateTimePicker.Visible = true;
					myDateTimePicker.ValueChanged +=
						new EventHandler(TimePickerValueChanged);
				}
				else
				{
					myDateTimePicker.Value = value;
					myDateTimePicker.Visible = false;
				}

				if (myDateTimePicker.Visible)
					DataGridTableStyle.DataGrid.Invalidate(bounds);
			}
		}

		protected override Size GetPreferredSize(
			Graphics g,
			object value)
		{
			return new Size(100, myDateTimePicker.PreferredHeight);
		}

		protected override int GetMinimumHeight()
		{
			return myDateTimePicker.PreferredHeight;
		}

		protected override int GetPreferredHeight(Graphics g,
		                                          object value)
		{
			return myDateTimePicker.PreferredHeight;
		}

		protected override void Paint(Graphics g,
		                              Rectangle bounds,
		                              CurrencyManager source,
		                              int rowNum)
		{
			Paint(g, bounds, source, rowNum, false);
		}

		protected override void Paint(
			Graphics g,
			Rectangle bounds,
			CurrencyManager source,
			int rowNum,
			bool alignToRight)
		{
			Paint(
				g, bounds,
				source,
				rowNum,
				Brushes.Red,
				Brushes.Blue,
				alignToRight);
		}

		protected override void Paint(
			Graphics g,
			Rectangle bounds,
			CurrencyManager source,
			int rowNum,
			Brush backBrush,
			Brush foreBrush,
			bool alignToRight)
		{
			try
			{
				CuteGridPaintCellEventArgs e
					= new CuteGridPaintCellEventArgs(g, bounds, source, this, rowNum, GetColumnValueAtRow(source, rowNum),
					                                 backBrush, foreBrush, alignToRight);

                e.SelectionBackBrush = new SolidBrush(this.DataGridTableStyle.DataGrid.SelectionBackColor);
                e.SelectionForeBrush = new SolidBrush(this.DataGridTableStyle.DataGrid.SelectionForeColor);

                ((CuteSoftelGrid)this.DataGridTableStyle.DataGrid).OnCellPaint(e);

				if (!e.CustomPaint)
				{
					if (this.DataGridTableStyle.DataGrid.IsSelected(rowNum))
					{
						backBrush = e.SelectionBackBrush;
						foreBrush = e.SelectionForeBrush;
					}
					else
					{
						backBrush = e.BackBrush;
						foreBrush = e.ForeBrush;
					}
//					DateTime date = (DateTime) 
//						GetColumnValueAtRow(source, rowNum);
					Rectangle rect = bounds;
					g.FillRectangle(backBrush, rect);
					rect.Offset(0, 2);
//					string strValue = date.ToString("d");
					string strValue = string.Empty;
					string strFormat = "{0:" + m_strFormat + "}";
					if (m_fpFormat == null)
						strValue = string.Format(strFormat, GetColumnValueAtRow(source, rowNum));
					else
						strValue = string.Format(m_fpFormat, strFormat, GetColumnValueAtRow(source, rowNum));
					int iWidth = g.MeasureString(strValue, this.DataGridTableStyle.DataGrid.Font).ToSize().Width;
					switch (m_haValueAlignment)
					{
						case HorizontalAlignment.Left:
							break;
						case HorizontalAlignment.Right:
							rect.X = rect.X + (rect.Width - iWidth);
							break;
						case HorizontalAlignment.Center:
							rect.X = rect.X + (rect.Width - iWidth)/2;
							break;
					}
					g.DrawString(strValue,
					             this.DataGridTableStyle.DataGrid.Font,
					             foreBrush, rect);
				}
			}
			catch (Exception)
			{
				base.Paint(g, bounds, source, rowNum, backBrush, foreBrush, alignToRight);
			}

		}

		protected override void SetDataGridInColumn(DataGrid value)
		{
			base.SetDataGridInColumn(value);
			if (myDateTimePicker.Parent != null)
			{
				myDateTimePicker.Parent.Controls.Remove
					(myDateTimePicker);
			}
			if (value != null)
			{
				value.Controls.Add(myDateTimePicker);
			}
		}

		private void TimePickerValueChanged(object sender, EventArgs e)
		{
			this.isEditing = true;
			base.ColumnStartedEditing(myDateTimePicker);
		}
	}
}
/********************* SniipSoftel *****************
 *
 *    $History: CuteGridDateTimeColumn.cs $
 *   
 *   *****************  Version 9  *****************
 *   User: Serg_ch      Date: 1.08.07    Time: 21:02
 *   Updated in $/Net/Common/ClientControl/CuteSoftelGrid/CuteGridColumns
 *   
 *   *****************  Version 8  *****************
 *   User: Serg_ch      Date: 30.07.07   Time: 21:04
 *   Updated in $/Net/Common/ClientControl/CuteSoftelGrid/CuteGridColumns
 *   
 *   *****************  Version 7  *****************
 *   User: Serg_ch      Date: 30.03.07   Time: 15:34
 *   Updated in $/Net/Common/ClientControl/CuteSoftelGrid/CuteGridColumns
 *   
 *   *****************  Version 5  *****************
 *   User: Autobuild    Date: 19.12.06   Time: 23:18
 *   Updated in $/Net/Common/SoftelControl/CuteSoftelGrid/CuteGridColumns
 *
 ********************* SniipSoftel *****************/
