/********************* SniipSoftel *****************
 *
 *   $Author: Serg_ch $
 *   $Revision: 1 $
 *   $Date: 13.08.07 16:12 $
 *
 ********************* SniipSoftel *****************/
using System;
using System.Drawing;
using System.Windows.Forms;
using Softel.Windows.Forms.CuteGridColumns;

namespace Softel.Windows.Forms
{
	/// <summary>
	/// Summary description for CuteGridLabelColumn.
	/// </summary>
	public class CuteGridLinkLabelColumn : DataGridColumnStyle
	{
		private LinkLabel m_lblEdit = new LinkLabel();
		private Brush m_brSelectionBackBrush;
		private Brush m_brSelectionForeBrush;
		private Brush m_brBackBrush;
		private Brush m_brForeBrush;
		private string m_strFormat;
		private IFormatProvider m_fpFormat;
		private HorizontalAlignment m_haValueAlignment;
		private EventHandler m_evtValueChanged;
		public event LinkLabelLinkClickedEventHandler LinkClicked;

		public CuteGridLinkLabelColumn()
		{
			m_strFormat = string.Empty;
			m_fpFormat = null;
			m_brSelectionBackBrush = null;
			m_brSelectionForeBrush = null;
			m_brBackBrush = null;
			m_brForeBrush = null;
			m_lblEdit.Visible = false;
			m_evtValueChanged = new EventHandler(TimePickerValueChanged);
			m_lblEdit.LinkClicked += new LinkLabelLinkClickedEventHandler(EventLinkClicked);
		}
		public void EventLinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			if(LinkClicked != null)
				LinkClicked(sender, e);
		}
		public HorizontalAlignment ValueAlignment
		{
			get { return m_haValueAlignment; }
			set { m_haValueAlignment = value; }
		}

		public string Format
		{
			get { return m_strFormat; }
			set { m_strFormat = value; }
		}

		public IFormatProvider FormatInfo
		{
			get { return m_fpFormat; }
			set { m_fpFormat = value; }
		}

		protected override Size GetPreferredSize(Graphics g, object value)
		{
			return g.MeasureString(value.ToString(), this.DataGridTableStyle.DataGrid.Font).ToSize();
		}

		protected override int GetMinimumHeight()
		{
			return this.DataGridTableStyle.DataGrid.Font.Height;
		}

		protected override int GetPreferredHeight(Graphics g, object value)
		{
			return g.MeasureString(value.ToString(), this.DataGridTableStyle.DataGrid.Font).ToSize().Height;
		}

		protected override void Abort(int rowNum)
		{
		}

		protected override bool Commit(CurrencyManager dataSource, int rowNum)
		{
			return true;
		}

		protected override void Edit(CurrencyManager source, int rowNum, Rectangle bounds, bool readOnly, string instantText, bool cellIsVisible)
		{
			object objValue = GetColumnValueAtRow(source, rowNum);
			m_lblEdit.Text = objValue.ToString();
			if (cellIsVisible)
			{
				m_lblEdit.Bounds = new Rectangle
					(bounds.X, bounds.Y,
					 bounds.Width, bounds.Height);
				m_lblEdit.Visible = true;
			}
			else
				m_lblEdit.Visible = false;

			if (m_lblEdit.Visible)
				DataGridTableStyle.DataGrid.Invalidate(bounds);		
		}

		protected override void Paint(Graphics g,
		                              Rectangle bounds,
		                              CurrencyManager source,
		                              int rowNum)
		{
			Paint(g, bounds, source, rowNum, false);
		}

		protected override void Paint(
			Graphics g,
			Rectangle bounds,
			CurrencyManager source,
			int rowNum,
			Brush backBrush,
			Brush foreBrush,
			bool alignToRight)
		{
			object objValue = GetColumnValueAtRow(source, rowNum);
			string linkText = "";
			if(objValue != null || objValue != DBNull.Value)
				linkText = objValue.ToString();
			
			m_lblEdit.Bounds = bounds;
			m_lblEdit.Parent = DataGridTableStyle.DataGrid;
//			m_lblEdit.Links.Add(0, linkText.Length, objValue);
			m_lblEdit.Text = linkText;
			m_lblEdit.Visible = true;
//			If (links.Contains(rowNum)) Then
//			m_link = CType(links(rowNum), LinkLabel)
//			If Not (m_link.Bounds.Equals(bounds)) Then
//			m_link.Bounds = bounds
//			m_link.Visible = True
//			End If
//			Else
//			m_link = New LinkLabel
//			m_link.Text = linkText
//			m_link.Parent = Me.
//			m_link.Links.Add(0, linkText.Length, o)
//			m_link.Bounds = bounds
//			AddHandler m_link.LinkClicked, AddressOf link_LinkClicked
//			links(rowNum) = m_link
//			End If			
//			if (m_brSelectionBackBrush == null)
//			{
//				m_brSelectionBackBrush = new SolidBrush(this.DataGridTableStyle.DataGrid.SelectionBackColor);
//				m_brSelectionForeBrush = new SolidBrush(this.DataGridTableStyle.DataGrid.SelectionForeColor);
//				m_brBackBrush = new SolidBrush(this.DataGridTableStyle.DataGrid.BackColor);
//				m_brForeBrush = new SolidBrush(this.DataGridTableStyle.DataGrid.ForeColor);
//			}
//
//			try
//			{
//				if (this.DataGridTableStyle.DataGrid.IsSelected(rowNum) || this.DataGridTableStyle.DataGrid.GetCurrentCellBounds() == bounds) //this.DataGridTableStyle.DataGrid.CurrentCell.RowNumber == rowNum)
//				{
//					backBrush = m_brSelectionBackBrush;
//					foreBrush = m_brSelectionForeBrush;
//				}
//
//				CuteGridPaintCellEventArgs e
//					= new CuteGridPaintCellEventArgs(g, bounds, source, this, rowNum, GetColumnValueAtRow(source, rowNum),
//					                                 backBrush, foreBrush, alignToRight);
//
//                e.SelectionBackBrush = new SolidBrush(this.DataGridTableStyle.DataGrid.SelectionBackColor);
//                e.SelectionForeBrush = new SolidBrush(this.DataGridTableStyle.DataGrid.SelectionForeColor);
//
//                ((CuteSoftelGrid)this.DataGridTableStyle.DataGrid).OnCellPaint(e);
//
//				if (!e.CustomPaint)
//				{
//					if (this.DataGridTableStyle.DataGrid.IsSelected(rowNum) || this.DataGridTableStyle.DataGrid.GetCurrentCellBounds() == bounds) //this.DataGridTableStyle.DataGrid.CurrentCell.RowNumber == rowNum)
//					{
//						backBrush = m_brSelectionBackBrush;
//						foreBrush = m_brSelectionForeBrush;
//					}
//					else
//					{
//						backBrush = e.BackBrush;
//						foreBrush = new SolidBrush(SystemColors.HotTrack);
//					}
//					string strValue = string.Empty;
//					string strFormat = "{0:" + m_strFormat + "}";
//					if (m_fpFormat == null)
//						strValue = string.Format(strFormat, GetColumnValueAtRow(source, rowNum));
//					else
//						strValue = string.Format(m_fpFormat, strFormat, GetColumnValueAtRow(source, rowNum));
//					Rectangle rect = bounds;
//					g.FillRectangle(backBrush, rect);
//					rect.Offset(0, 2);
//					int iWidth = g.MeasureString(strValue, this.DataGridTableStyle.DataGrid.Font).ToSize().Width;
//					switch (m_haValueAlignment)
//					{
//						case HorizontalAlignment.Left:
//							break;
//						case HorizontalAlignment.Right:
//							rect.X = rect.X + (rect.Width - iWidth);
//							break;
//						case HorizontalAlignment.Center:
//							rect.X = rect.X + (rect.Width - iWidth)/2;
//							break;
//					}
//					Font fntLink = new Font(this.DataGridTableStyle.DataGrid.Font.FontFamily, this.DataGridTableStyle.DataGrid.Font.Size, FontStyle.Italic | FontStyle.Underline);
//					g.DrawString(strValue,
//					             fntLink,
//					             foreBrush, rect);
//				}
//			}
//			catch (Exception)
//			{
//				base.Paint(g, bounds, source, rowNum, backBrush, foreBrush, alignToRight);
//			}
		}

		protected override void Paint(Graphics g, Rectangle bounds, CurrencyManager source, int rowNum, bool alignToRight)
		{
			Paint(
				g, bounds,
				source,
				rowNum,
				m_brBackBrush,
				m_brForeBrush,
				alignToRight);
		}
		private void TimePickerValueChanged(object sender, EventArgs e)
		{
			base.ColumnStartedEditing(m_lblEdit);
		}
	}
}
/********************* SniipSoftel *****************
 *
 *    $History: CuteGridLinkLabelColumn.cs $
 *   
 *   *****************  Version 1  *****************
 *   User: Serg_ch      Date: 13.08.07   Time: 16:12
 *   Created in $/Net/Common/ClientControl/CuteSoftelGrid/CuteGridColumns
 *   
 *   *****************  Version 8  *****************
 *   User: Serg_ch      Date: 1.08.07    Time: 21:02
 *   Updated in $/Net/Common/ClientControl/CuteSoftelGrid/CuteGridColumns
 *   
 *   *****************  Version 7  *****************
 *   User: Serg_ch      Date: 30.07.07   Time: 21:04
 *   Updated in $/Net/Common/ClientControl/CuteSoftelGrid/CuteGridColumns
 *   
 *   *****************  Version 5  *****************
 *   User: Autobuild    Date: 19.12.06   Time: 23:18
 *   Updated in $/Net/Common/SoftelControl/CuteSoftelGrid/CuteGridColumns
 *
 ********************* SniipSoftel *****************/
