/********************* SniipSoftel *****************
 *
 *   $Author: Serg_ch $
 *   $Revision: 14 $
 *   $Date: 1.08.07 21:02 $
 *
 ********************* SniipSoftel *****************/
using System;
using System.Drawing;
using System.Windows.Forms;
using Softel.Windows.Forms.CuteGridColumns;

namespace Softel.Windows.Forms
{
	/// <summary>
	/// Summary description for CuteGridTextBoxColumn.
	/// </summary>
	public class CuteGridTextBoxColumn : DataGridTextBoxColumn
	{
		private DataGrid DG = null;
		private CurrencyManager m_cmSource = null;
		private string m_strFormat;
		private IFormatProvider m_fpFormat;
		private HorizontalAlignment m_haValueAlignment;
		//		private int m_iPos = 0;
		public CuteGridTextBoxColumn()
		{
			TextBox.Enter += new EventHandler(TextBox_Enter);
			TextBox.WordWrap = false;
		}


		private void Init()
		{
            if (this.DataGridTableStyle != null)
			{
				DataGrid dgTemp = this.DataGridTableStyle.DataGrid;
                if (dgTemp.DataSource != null)
                {
                    CurrencyManager cmTemp = (CurrencyManager) dgTemp.BindingContext[dgTemp.DataSource, dgTemp.DataMember];
                    if(m_cmSource == null || m_cmSource != cmTemp)
                    {
                        DG = dgTemp;
                        m_cmSource = (CurrencyManager) DG.BindingContext[DG.DataSource, DG.DataMember];
                        m_cmSource.PositionChanged += new EventHandler(TextBox_Enter);
                    }
                }
			}
		}

		public HorizontalAlignment ValueAlignment
		{
			get { return m_haValueAlignment; }
			set { m_haValueAlignment = value; }
		}

		public string Format
		{
			get { return m_strFormat; }
			set { m_strFormat = value; }
		}

		public IFormatProvider FormatInfo
		{
			get { return m_fpFormat; }
			set { m_fpFormat = value; }
		}


		// Paint    background'a
		protected override void Paint(Graphics g, Rectangle bounds, CurrencyManager source,
		                              int rowNum, Brush backBrush, Brush foreBrush, bool alignToRight)
		{
			//this.Paint(
			Init();
			try
			{
				CuteGridPaintCellEventArgs e
					= new CuteGridPaintCellEventArgs(g, bounds, source, this, rowNum, GetColumnValueAtRow(source, rowNum),
					                                 backBrush, foreBrush, alignToRight);
                e.SelectionBackBrush = new SolidBrush(this.DataGridTableStyle.DataGrid.SelectionBackColor);
                e.SelectionForeBrush = new SolidBrush(this.DataGridTableStyle.DataGrid.SelectionForeColor);

				((CuteSoftelGrid) this.DataGridTableStyle.DataGrid).OnCellPaint(e);

				if (!e.CustomPaint)
				{
					if (this.DataGridTableStyle.DataGrid.IsSelected(rowNum))
					{
						backBrush = e.SelectionBackBrush;
						foreBrush = e.SelectionForeBrush;
					}
					else
					{
						backBrush = e.BackBrush;
						foreBrush = e.ForeBrush;
					}
					string strValue = string.Empty;
					string strFormat = "{0:" + m_strFormat + "}";
					if (m_fpFormat == null)
						strValue = string.Format(strFormat, GetColumnValueAtRow(source, rowNum));
					else
						strValue = string.Format(m_fpFormat, strFormat, GetColumnValueAtRow(source, rowNum));
//					base.Paint(g, bounds, source, rowNum, backBrush, foreBrush, alignToRight);
					Rectangle rect = bounds;
					g.FillRectangle(backBrush, rect);
					rect.Offset(0, 2);
					int iWidth = g.MeasureString(strValue, this.DataGridTableStyle.DataGrid.Font).ToSize().Width;
					switch (m_haValueAlignment)
					{
						case HorizontalAlignment.Left:
							break;
						case HorizontalAlignment.Right:
							rect.X = rect.X + (rect.Width - iWidth);
							break;
						case HorizontalAlignment.Center:
							rect.X = rect.X + (rect.Width - iWidth)/2;
							break;
					}
					g.DrawString(strValue,
					             this.DataGridTableStyle.DataGrid.Font,
					             foreBrush, rect);
				}
			}
			catch (Exception)
			{
				base.Paint(g, bounds, source, rowNum, backBrush, foreBrush, alignToRight);
			}

		}

		private void TextBoxRedraw()
		{
			Init();
			if ((m_cmSource.Position/2)*2 == m_cmSource.Position)
			{
				TextBox.BackColor = this.DataGridTableStyle.BackColor;
				TextBox.ForeColor = this.DataGridTableStyle.ForeColor;
			}
			else
			{
				TextBox.BackColor = this.DataGridTableStyle.AlternatingBackColor;
				TextBox.ForeColor = this.DataGridTableStyle.ForeColor;
			}

            //int rowNum = m_cmSource.Position;
            //Graphics g = Graphics.FromHwnd(DG.Handle);
            //Rectangle rect = DG.GetCellBounds(DG.CurrentCell);

            //CuteGridPaintCellEventArgs e
            //    = new CuteGridPaintCellEventArgs(g, rect, m_cmSource, this, rowNum, GetColumnValueAtRow(m_cmSource, rowNum),
            //                                     TextBox.BackColor, TextBox.ForeColor);

            //((CuteSoftelGrid) this.DataGridTableStyle.DataGrid).OnCellPaint(e);
            //TextBox.BackColor = e.TextBoxBackColor;
            //TextBox.ForeColor = e.TextBoxForeColor;
            //if (e.CustomPaint /* && this.ReadOnly*/)
            //    TextBox.Visible = false;
		}

		private void TextBox_Enter(object sender, EventArgs e)
		{
			TextBoxRedraw();
		}
	}
}
/********************* SniipSoftel *****************
 *
 *    $History: CuteGridTextBoxColumn.cs $
 *   
 *   *****************  Version 14  *****************
 *   User: Serg_ch      Date: 1.08.07    Time: 21:02
 *   Updated in $/Net/Common/ClientControl/CuteSoftelGrid/CuteGridColumns
 *   
 *   *****************  Version 13  *****************
 *   User: Serg_ch      Date: 31.07.07   Time: 20:58
 *   Updated in $/Net/Common/ClientControl/CuteSoftelGrid/CuteGridColumns
 *   
 *   *****************  Version 12  *****************
 *   User: Serg_ch      Date: 30.07.07   Time: 21:04
 *   Updated in $/Net/Common/ClientControl/cutesoftelgrid/cutegridcolumns
 *   
 *   *****************  Version 10  *****************
 *   User: Autobuild    Date: 19.12.06   Time: 23:18
 *   Updated in $/Net/Common/SoftelControl/CuteSoftelGrid/CuteGridColumns
 *
 ********************* SniipSoftel *****************/
