/********************* SniipSoftel *****************
 *
 *   $Author: Serg_ch $
 *   $Revision: 38 $
 *   $Date: 2.08.07 16:41 $
 *
 ********************* SniipSoftel *****************/
/// Project: SoftelControl, File: CuteSoftelGrid.cs</p>
/// Namespace: Softel.Windows.Forms, Class: CuteSoftelGrid</p>
/// Creation date: 28.12.2005 17:58</p>
/// Last modified: 24.03.2006 8:15</p>

using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Design;
using System.Reflection;
using System.Windows.Forms;
using Softel.MiddleTier.Client;
using Softel.Windows.Forms.CuteGridColumns;
using Softel.Windows.Forms.CuteTableStyles;

namespace Softel.Windows.Forms
{
	/// <summary>
	/// Summary description for CuteSoftelGrid.
	/// </summary>
	public class CuteSoftelGrid : DataGrid
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private Container components = null;


		/// <summary>
		///    
		/// </summary>
		private bool m_bHighlightRow;

		/// <summary>
		///    
		/// </summary>
		private String m_strTableName = null;

		private Hashtable m_htCondition=null;


		public delegate void CuteGridPaintCellEventHandler(object sender, CuteGridPaintCellEventArgs e);

		public event CuteGridPaintCellEventHandler CellPaint;

		public virtual void OnCellPaint(
			CuteGridPaintCellEventArgs e)
		{
			if (CellPaint != null)
				CellPaint(this, e);
		}


		public CuteSoftelGrid()
		{
			InitializeComponent();
			this.CaptionBackColor = Color.Gainsboro;
			this.CaptionForeColor = Color.Black;
			this.HighlightRow = true;
			this.MouseUp += new MouseEventHandler(CuteSoftelGridMouseUp);
		}
		public ScrollBar HorizontalScrollBar
		{
			get{ return HorizScrollBar;}
		}

		/// <summary>
///    MouseUp    
/// </summary>
/// <param name="sender"></param>
/// <param name="e"></param>
		private void CuteSoftelGridMouseUp(object sender, MouseEventArgs e)
		{
			Point pt = new Point(e.X, e.Y);
			HitTestInfo hti = this.HitTest(pt);
			if (hti.Row >= 0 && hti.Column >= 0)
			{
				if (this.HighlightRow == true) // only highlight if setting turned on
				{
					if (hti.Type == HitTestType.Cell)
					{
						this.CurrentCell = new DataGridCell(hti.Row, hti.Column);

						this.Select(hti.Row);
					}
				}
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (components != null)
					components.Dispose();
			}
			base.Dispose(disposing);
		}

		#region Component Designer generated code

		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			((System.ComponentModel.ISupportInitialize) (this)).BeginInit();
			((System.ComponentModel.ISupportInitialize) (this)).EndInit();

		}

		#endregion

		[Editor(typeof (CuteGridTableStylesCollectionEditor), typeof (UITypeEditor))]
		new public GridTableStylesCollection TableStyles
		{
			get { return base.TableStyles; }
		}
	    public DataRow[] SelectedRows
	    {
	        get
	        {
                ArrayList alSelectedRow = new ArrayList();
                DataView dataView = (DataView)((CurrencyManager)BindingContext[this.DataSource]).List;
                int rowCount = dataView.Count;
                for (int rowIndex = 0; rowIndex < rowCount; rowIndex++)
                {
                    if(this.IsSelected(rowIndex))
                        alSelectedRow.Add(dataView[rowIndex].Row);
                }
	            return (DataRow[]) alSelectedRow.ToArray(typeof(DataRow));
	        }
	    }
		/// <summary>
///     
/// </summary>
/// <param name="strFieldName">  </param>
/// <param name="strValue">  </param>
		public void SelectRow(string strFieldName, string strValue)
		{
			int iRowCount;
			int iColIndex = -1;

			BindingManagerBase objManager = this.BindingContext[this.DataSource, this.DataMember];
			iRowCount = objManager.Count;

			for (int i = 0; i < this.TableStyles[0].GridColumnStyles.Count; i++)
				//foreach(DataColumn drColumn in ((DataTable)grdElement.DataSource).Columns)
			{
				DataGridColumnStyle dc = TableStyles[0].GridColumnStyles[i];
				if (strFieldName.ToUpper() == dc.MappingName.ToString().ToUpper())
				{
					iColIndex = i;
					break;
				}
			}


			if (iColIndex >= 0)
			{
				int iPos = objManager.Position;


				for (int iRowCtr = 0; iRowCtr < iRowCount; iRowCtr++)
				{
					string strCurrentValue = this[iRowCtr, iColIndex].ToString();
					if (strCurrentValue == strValue)
					{
						objManager.Position = iRowCtr;
						this.UnSelect(iPos);
						this.UnSelect(iRowCtr);
						this.Select(iRowCtr);
						break;
					}
				}


			}

		}

		public DataRowView CurrentRowView
		{
			get
			{
                if (this.DataSource != null)
                {
                    foreach(DictionaryEntry objC in this.BindingContext)
                    {
                        if(((WeakReference) objC.Value).Target is CurrencyManager)
                        {
                            CurrencyManager cmTheme = (CurrencyManager) ((WeakReference) objC.Value).Target;
                            if(cmTheme.Count > 0)
                            {
                                DataRowView drvTheme = (DataRowView) cmTheme.Current;
                                if(this.DataSource.Equals(drvTheme.Row.Table))
                                    return drvTheme;
                            }
                        }
                    }
                }
			    return null;
			}
		}

		public DataRow CurrentRow
		{
			get
			{
				if(CurrentRowView == null)
					return null;
				else
					return CurrentRowView.Row;
			}
		}

		/// <summary>
		///    datatable
		/// </summary>
		public DataTable GridTableStyleTable
		{
			set
			{
				DataGridTableStyle tsGrid;
				int iWidth;
				if (this.TableStyles.Count > 0)
					this.TableStyles.RemoveAt(0);
					//tsGrid = this.TableStyles[0];
				//else
				{
					//this.TableStyles[0].Dispose();
					
					tsGrid = new DataGridTableStyle();
					if (m_strTableName == null)
						tsGrid.MappingName = "Table";
					else
						tsGrid.MappingName = m_strTableName;
				}
				DataTable dtComboBox = null;
				int iColCount = 0;
				int iColIndex = 0;
				int iCount = value.Rows.Count;
				int iIndex = 0;
				while(iIndex < iCount)
				{
					DataRow drCurrent = value.Rows[iIndex++];
					bool bAdd=true;
					if(m_htCondition !=null && value.Columns.Contains("condition"))
					{
						if(drCurrent["condition"] !=DBNull.Value )
						{
							string strCondition = drCurrent["condition"].ToString().Trim();
							if( strCondition !=null)
								if(strCondition.Split('=').Length ==2)
								{
									string[] strCond=strCondition.Split('=');
									if(m_htCondition.Contains(strCond[0]))
									{
										string strCurrentValue = Convert.ToInt32(m_htCondition[strCond[0]]).ToString();
										if(strCurrentValue!=strCond[1])
											bAdd = false;
									}
								}
						}
					}
					if (!bAdd)
						continue;
					DataGridColumnStyle dcNew;
					string strField = drCurrent["field"].ToString();

					if (drCurrent["visible"] != null && drCurrent["visible"] != DBNull.Value)
						if (!Convert.ToBoolean(drCurrent["visible"]) )
							continue;
					if (this.DataSource != null && this.DataSource is DataTable && ((DataTable) this.DataSource).Columns[strField] != null)
					{
						string strTypeName = ((DataTable) this.DataSource).Columns[strField].DataType.Name;
						if (strTypeName == "Boolean")
							dcNew = new CuteGridBoolColumn();
						else if (strTypeName == "String" && !Convert.ToBoolean(drCurrent["read_only"]))
						{
							if(drCurrent["StoredProc"] != DBNull.Value)
							{

								string[] strParams =drCurrent["StoredProc"].ToString().Split('|');
								string strProcName=strParams[0];
								using(Command cmdComboBox = new Command())
								{
									cmdComboBox.CommandName = strProcName;
									if( strParams.Length > 1 && m_htCondition!=null)
									{
										
										string strName=strParams[1];
										cmdComboBox.InputParams["@"+strName]=m_htCondition[strName];

									}
									cmdComboBox.Execute(true);
									dtComboBox = cmdComboBox.DataSet.Tables[0];
								}
//								}
								dcNew = new CuteGridComboBoxColumn();
								((CuteGridComboBoxColumn) dcNew).DataSource = dtComboBox;
								((CuteGridComboBoxColumn) dcNew).DataValue = drCurrent["DataValue"].ToString();
								((CuteGridComboBoxColumn)dcNew).DataKey = drCurrent["DataKey"].ToString();
							}
							else 
								dcNew = new CuteGridTextBoxColumn();
						}
						else if (strTypeName == "DateTime" && !Convert.ToBoolean(drCurrent["read_only"]))
							dcNew = new CuteGridDateTimeColumn();
						else
							dcNew = new CuteGridLabelColumn();

					}
					else
						dcNew = new CuteGridLabelColumn();
					dcNew.ReadOnly = Convert.ToBoolean(drCurrent["read_only"]);
					dcNew.MappingName = strField;
					dcNew.HeaderText = drCurrent["caption"].ToString();
					if (drCurrent["width"] != DBNull.Value)
						iWidth = Convert.ToInt32(drCurrent["width"]);
					else
						iWidth = 0;
					if (iWidth > 0)
						dcNew.Width = iWidth;
					else
						dcNew.Width = dcNew.HeaderText.Length*10;
					dcNew.NullText = string.Empty;
					dcNew.Alignment = HorizontalAlignment.Center;
					bool bExists = false;
					iColCount = tsGrid.GridColumnStyles.Count;
					iColIndex = 0;
					while(iColIndex < iColCount)
					{
						DataGridColumnStyle tsSearch = tsGrid.GridColumnStyles[iColIndex++];
					
						if (tsSearch.MappingName == dcNew.MappingName)
						{
							bExists = true;
							break;
						}
					}
					if (!bExists)
						tsGrid.GridColumnStyles.Add(dcNew);
				}
				this.TableStyles.Add(tsGrid);
			}
		}

		/// <summary>
///          
/// </summary>
		public bool HighlightRow
		{
			set { m_bHighlightRow = value; }
			get { return m_bHighlightRow; }
		}

		/// <summary>
///      
/// </summary>

		#region SetDataGridTableStyle (     )
		public void SetDataGridTableStyle(string strColumnTypeProcedure, string strColumnType)
		{
			SetDataGridTableStyle(strColumnTypeProcedure, strColumnType, null,null);


		}
		public void SetDataGridTableStyle(string strColumnTypeProcedure, string strColumnType, string strTableName)
		{
			SetDataGridTableStyle(strColumnTypeProcedure, strColumnType, strTableName,null);


		}

		public void SetDataGridTableStyle(string strColumnTypeProcedure, string strColumnType, string strTableName,Hashtable htParams)
		{
			Command cmdProc = new Command();
			cmdProc.CommandName = strColumnTypeProcedure;
			cmdProc.InputParams["@column_type"] = strColumnType; //deprecated
			cmdProc.InputParams["@dict_name"] = strColumnType;
			cmdProc.Execute(true);
			m_strTableName = strTableName;
			m_htCondition = htParams;
			
			
			
			GridTableStyleTable = cmdProc.DataSet.Tables[0];
			
	

			if (cmdProc.DataSet.Tables.Count > 1)
			{
				DataRow drCaption = cmdProc.DataSet.Tables[1].Rows[0];
				bool bCaptionVisible = Convert.ToBoolean(drCaption["caption_visible"]);
				this.CaptionVisible = bCaptionVisible;
				if (bCaptionVisible)
					this.CaptionText = Convert.ToString(drCaption["caption"]);
			} // if

		}

		#endregion

		#region ColumnAutoResize (     )

		/// <summary>
		///      
		/// </summary>
		/// <param name="columnIndex"> ,    </param>
		public void ColumnAutoResize(int columnIndex)
		{
			try
			{
				MethodInfo methodInfo = typeof
					(DataGrid).GetMethod("ColAutoResize",
					                     BindingFlags.Instance |
					                     	BindingFlags.NonPublic |
					                     	BindingFlags.InvokeMethod);
				methodInfo.Invoke(this, new object[] {columnIndex});
			}
			catch (Exception)
			{
			}
		}

		/// <summary>
		///       
		/// </summary>
		public void ColumnAutoResize()
		{
			for (int k = 0; k < this.TableStyles.Count; k++)
			{
				DataGridTableStyle ts = this.TableStyles[k];

				if (ts != null)
				{
					for (int i = 0; i < ts.GridColumnStyles.Count; i++)
						// if(ts.GridColumnStyles[i].Width!=0)
						ColumnAutoResize(i);

				}
			}
		}

		#endregion
	}
}
/********************* SniipSoftel *****************
 *
 *    $History: CuteSoftelGrid.cs $
 *   
 *   *****************  Version 38  *****************
 *   User: Serg_ch      Date: 2.08.07    Time: 16:41
 *   Updated in $/Net/Common/ClientControl/CuteSoftelGrid
 *   
 *   *****************  Version 37  *****************
 *   User: Serg_ch      Date: 31.07.07   Time: 20:58
 *   Updated in $/Net/Common/ClientControl/CuteSoftelGrid
 *   
 *   *****************  Version 36  *****************
 *   User: Serg_ch      Date: 27.07.07   Time: 21:01
 *   Updated in $/Net/Common/ClientControl/CuteSoftelGrid
 *   
 *   *****************  Version 34  *****************
 *   User: Serg_ch      Date: 22.02.07   Time: 13:42
 *   Updated in $/Net/Common/SoftelControl/CuteSoftelGrid
 *   
 *   *****************  Version 33  *****************
 *   User: Autobuild    Date: 19.12.06   Time: 23:18
 *   Updated in $/Net/Common/SoftelControl/CuteSoftelGrid
 *
 ********************* SniipSoftel *****************/
