/********************* SniipSoftel *****************
 *
 *   $Author: Gavrilov $
 *   $Revision: 61 $
 *   $Date: 21.05.07 17:52 $
 *
 ********************* SniipSoftel *****************/
/// Project: SoftelControl, File: ParamsDialog.cs</p>
/// Namespace: Softel.Common, Class: ParamsDialog</p>
/// Creation date: 28.12.2005 17:58</p>
/// Last modified: 23.03.2006 20:21</p>

using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Windows.Forms;
using FirebirdSql.Data.Firebird;
using Softel.MiddleTier.Client;
using Softel.Windows.Forms;
using MessageBox = Softel.Windows.Forms.MsgBox;

namespace Softel.Common
{
	/// <summary>
	///      
	/// </summary>
	/// <example>
	///   
	/// 
	/// <code>
	///	ParamsDialog pmGroup = new ParamsDialog();
	///	pmGroup.Params.Add(new Param(" ", ParamsType.String, "", "  ", " "));
	///	pmGroup.Params.Add(new Param(" ", ParamsType.String, "", "  ", "  "));
	///
	///	if(pmGroup.ShowDialog() == DialogResult.OK)
	///	{
	///		SSP sspGroup = new SSP();
	///		sspGroup.Connection = m_sqlConnection;
	///		sspGroup.Clear();
	///		sspGroup.Params["@RID"]			= 0;
	///		sspGroup.Params["@Name"]		= pmGroup.Value[" "].ToString();
	///		sspGroup.Params["@Description"]	= pmGroup.Value[" "].ToString();
	///		sspGroup.Params["@IsCustom"]	= true;
	///		sspGroup.Execute("admin_insGroup");
	///		return true;
	///	}
	/// </code>
	/// </example>
	public class ParamsDialog : Form
	{
		/// <summary>,       </summary>
		public event ParamsDialogEventHandler OnGetValue;

		/// <summary>,      </summary>
		public event ParamsDialogEventHandler OnSetValue;

		/// <summary>,     </summary>
		public event ParamsDialogEventHandler OnAfterSetValue;

		/// <summary>
		/// . <see cref="System.ComponentModel.Container"/>
		/// </summary>
		private Container components = null;

		/// <summary>
		///    
		/// </summary>
		private PropertyGrid m_objPropertyGrid;

		/// <summary>
		///  ParamsDialog.     CustomForm     SoftelWizard
		/// </summary>
		private ParamsDialogCollection m_objParamsDialogCollection;

		/// <summary>
		///   
		/// </summary>
		private ParamCollection m_objParamCollection;

		/// <summary>
		///      <see cref="System.Windows.Forms.PropertyGrid"/>
		/// </summary>
		private ParamBag m_objParamBag;

		/// <summary>
		///  ""
		/// </summary>
		private ButtonXP m_btnCancel;

		/// <summary>
		///  ""
		/// </summary>
		private ButtonXP m_btnOk;

		/// <summary>
		/// -   
		/// </summary>
		private Hashtable m_htValue;

		private Hashtable m_htInsertValue;
		private Hashtable m_htUpdateValue;
		private string m_strInsertStoredProc;
		private string m_strUpdateStoredProc;
		private bool m_bRestoreDefault;
        //private Softel.Windows.Forms.Button.ControlButtons controlButtons1;
		private ParamEventArgs m_parOnce;

		/// <summary>
		///     
		/// </summary>
		/// <remarks>
		///       ,     
		///      <see cref="Params"/>    <see cref="Param"/>
		/// </remarks>
		public ParamsDialog()
		{
			m_objParamCollection = new ParamCollection();
			m_objParamBag = new ParamBag();
			m_htValue = new Hashtable();
			m_htInsertValue = new Hashtable();
			m_htUpdateValue = new Hashtable();
			m_bRestoreDefault = true;
			InitializeComponent();
			m_objPropertyGrid.ParentChanged += new EventHandler(m_objPropertyGrid_ParentChanged);
			SizeGripStyle = SizeGripStyle.Hide;
			m_objPropertyGrid.PropertySort = PropertySort.Categorized;

		}

		/// <summary>
		///       
		/// RecId, Description, Value
		/// </summary>
		/// <param name="dtParams">
		/// 	<para>
		///           <see cref="DataTable"/>,     
		///         
		///     </para>
		/// 	<list type="number">
		/// 		<item>
		/// 			<div style="MARGIN-RIGHT: 0px">
		///                 RecId
		///             </div>
		/// 		</item>
		/// 		<item>
		/// 			<div style="MARGIN-RIGHT: 0px">
		///                 Description
		///             </div>
		/// 		</item>
		/// 		<item>
		/// 			<div style="MARGIN-RIGHT: 0px">
		///                 Value
		///             </div>
		/// 		</item>
		/// 	</list>
		/// </param>
		public ParamsDialog(DataTable dtParams) : this()
		{
			Param pmTemp;
			int iCount = dtParams.Rows.Count;
			int iIndex = 0;
			while(iIndex < iCount)
			{
				DataRow drCurrent = dtParams.Rows[iIndex++];
				pmTemp = new Param(drCurrent["RecId"].ToString(), drCurrent["Description"].ToString(), ParamsType.String, null, null, drCurrent["Value"]);
				this.Params.Add(pmTemp);
			}
		}

		/// <summary>    </summary>
		/// <remarks>
		/// 	<blockquote dir="ltr" style="MARGIN-RIGHT: 0px">
		/// 		<para align="center"><font color="black" size="1"> ,  
		///           ,     
		///          :</font></para>
		/// 	</blockquote>
		/// 	<para class="xmldocbulletlist"><font color="black" size="1"><u>
		///     </u></font></para>
		/// 	<list type="bullet">
		/// 		<item><font color="black">[<font size="1">caption</font><font size="1">]
		///         [</font><font size="1">varchar]</font><font size="1">(</font><font size="1">255</font>
		/// 				<font size="1">) -    </font></font></item>
		/// 		<item><font color="black">[<font size="1">desk</font><font size="1">]
		///         [</font><font size="1">varchar]</font><font size="1">(</font><font size="1">255</font>
		/// 				<font size="1">) -    </font></font></item>
		/// 		<item><font color="black">[<font size="1">order]</font><font size="1">[</font>
		/// 				<font size="1">int] -   </font></font></item>
		/// 		<item><font color="black">[<font size="1">readonly</font><font size="1">]
		///         [</font><font size="1">bit] - ,    
		///         </font></font></item>
		/// 		<item><font color="black">[<font size="1">defaultvalue</font><font size="1">]
		///         [</font><font size="1">varchar]</font><font size="1">(</font><font size="1">50</font>
		/// 				<font size="1">) -   </font></font></item>
		/// 		<item><font color="black">[<font size="1">must_be_filled</font><font size="1">]
		///         [</font><font size="1">bit] ,     
		///          </font></font></item>
		/// 		<item><font color="black">[<font size="1">data_type</font><font size="1">]
		///         [</font><font size="1">varchar]</font><font size="1">(</font><font size="1">50</font>
		/// 				<font size="1">) -   </font></font></item>
		/// 		<item><font color="black">[<font size="1">category</font><font size="1">]
		///         [</font><font size="1">varchar]</font><font size="1">(</font><font size="1">255</font>
		/// 				<font size="1">) -       
		///         </font></font></item>
		/// 	</list>
		/// 	<para class="xmldocbulletlist"><font color="black" size="1"><u>  
		///        (  - Data)</u></font></para>
		/// 	<list type="bullet">
		/// 		<item><font color="black">[<font size="1">storedproc</font><font size="1">]
		///         [</font><font size="1">varchar] -     
		///         </font></font></item>
		/// 		<item><font color="black">[<font size="1">datakey</font><font size="1">]
		///         [</font><font size="1">varchar]</font><font size="1">(</font><font size="1">50</font>
		/// 				<font size="1">) -  </font></font></item>
		/// 		<item><font color="black">[<font size="1">datavalue</font><font size="1">]
		///         [</font><font size="1">varchar]</font><font size="1">(</font><font size="1">50)
		///         -      </font></font></item>
		/// 	</list>
		/// 	<para class="xmldocbulletlist" dir="ltr" style="MARGIN-RIGHT: 0px">
		/// 		<font color="black" size="1"><u>   List</u></font></para>
		/// 	<list type="bullet">
		/// 		<item><font color="black">[<font size="1">list</font><font size="1">]
		///         [</font><font size="1">nvarchar]</font><font size="1">(</font><font size="1">1000</font>
		/// 				<font size="1">) -    
		///         '|'</font></font></item>
		/// 		<item><font color="black">[<font size="1">list_type</font><font size="1">]
		///         [</font><font size="1">bit] - ,    -  
		///         </font></font></item>
		/// 	</list>
		/// 	<para class="xmldocbulletlist"><font color="black" size="1"><u> 
		///        </u></font></para>
		/// 	<list type="bullet">
		/// 		<item><font color="black">[<font size="1">IsNew</font><font size="1">]
		///         [</font><font size="1">bit] -     
		///         </font></font></item>
		/// 		<item><font color="black">[<font size="1">StoredProcParam</font>
		/// 				<font size="1">] -       
		///          </font></font></item>
		/// 		<item><font color="black">[<font size="1">ParamTypeParam</font>
		/// 				<font size="1">] [</font><font size="1">int] -    
		///            </font></font></item>
		/// 	</list>
		/// 	<para class="xmldocbulletlist"><font size="1"><u>
		///     </u></font></para>
		/// 	<list type="bullet">
		/// 		<item><font size="1">param_id</font><font size="1">]
		///         [</font><font size="1">int] -   </font></item>
		/// 		<item><font color="black">[<font size="1">field</font><font size="1">]
		///         [</font><font size="1">varchar]</font><font size="1">(</font><font size="1">50</font>
		/// 				<font size="1">) - ,   
		///         </font></font></item>
		/// 		<item><font color="black">[<font size="1">upd_proc_param</font><font size="1">]
		///         [</font><font size="1">varchar]</font><font size="1">(</font><font size="1">50</font>
		/// 				<font size="1">) -     
		///         </font></font></item>
		/// 		<item><font color="black">[<font size="1">ins_proc_param</font><font size="1">]
		///         [</font><font size="1">varchar]</font><font size="1">(</font><font size="1">50</font>
		/// 				<font size="1">) -     
		///         </font></font></item>
		/// 		<item><font color="black">[<font size="1">param_type</font><font size="1">]
		///         [</font><font size="1">int] -   (. )</font></font></item>
		/// 		<item><font color="black">[<font size="1">page_id</font><font size="1">]
		///         [</font><font size="1">int] -  </font></font></item>
		/// 		<item><font color="black">[<font size="1">max_value</font><font size="1">]
		///         [</font><font size="1">sql_variant</font><font size="1">] - 
		///         </font></font></item>
		/// 		<item><font color="black">[<font size="1">min_value</font><font size="1">]
		///         [</font><font size="1">sql_variant</font><font size="1">] - 
		///         </font></font></item>
		/// 	</list>
		/// 	<para class="xmldocbulletlist"></para>
		/// 	<para class="xmldocbulletlist" align="center"><font size="1"> , 
		///        ,   
		///     </font></para>
		/// 	<para class="xmldocbulletlist" align="center"></para>
		/// 	<div class="xmldocbulletlist" align="left">
		/// 		<list type="bullet">
		/// 			<item>
		/// 				<font color="black">[</font><font color="black"><font size="1">ParamTypeId</font>
		/// 					<font size="1">] [</font><font size="1">int] -  
		///              </font></font></item>
		/// 			<item>
		/// 				<font color="black">[</font><font color="black"><font size="1">Name</font>
		/// 					<font size="1">] [</font><font size="1">varchar]</font>
		/// 					<font size="1">(255) -   </font></font></item>
		/// 			<item>
		/// 				<font color="black">[</font><font color="black"><font size="1">InsertCommand</font>
		/// 					<font size="1">] [</font><font size="1">varchar]</font>
		/// 					<font size="1">(</font><font size="1">255) -    
		///             </font></font></item>
		/// 			<item>
		/// 				<font color="black">[</font><font color="black"><font size="1">UpdateCommand</font>
		/// 					<font size="1">] [</font><font size="1">varchar]</font>
		/// 					<font size="1">(</font><font size="1">255</font><font size="1">)</font>
		/// 					<font size="1">-    
		///             </font></font></item>
		/// 			<item>
		/// 				<font color="black">[</font><font color="black"><font size="1">Caption</font>
		/// 					<font size="1">] [</font><font size="1">varchar]</font>
		/// 					<font size="1">(</font><font size="1">255</font><font size="1">)</font>
		/// 					<font size="1">  </font></font></item>
		/// 		</list>
		/// 	</div>
		/// 	<para class="xmldocbulletlist"></para>
		/// 	<para class="xmldocbulletlist"><font size="1">   
		///      , <see cref="ParamsDialog"/>    
		///       .</font></para>
		/// </remarks>
		/// <param name="dtParams">   </param>
		/// <param name="strStoredProc"> ,   </param>
		/// <param name="strParamType">  </param>
		public ParamsDialog(DataTable dtParams, string strStoredProc, string strParamType) : this()
		{
			Command cmdProc = new Command();
			cmdProc.CommandName = strStoredProc;
			cmdProc.InputParams["@Param_Type"] = strParamType;
			cmdProc.InputParams["@dict_name"] = strParamType;
			cmdProc.Execute(true);
			if (cmdProc.DataSet.Tables[0].Rows.Count > 0)
				InitDialog(dtParams, cmdProc.DataSet.Tables[0], null);
		}

		/// <remarks>
		/// 	<para>
		///              
		///         <see cref="ParamsDialog(System.Data.DataTable,System.String,System.String)">ParamsDialog(System.Data.DataTable,
		///         string, string) Method</see>
		/// 	</para>
		/// </remarks>
		/// <summary>    </summary>
		/// <param name="dtParams">   </param>
		/// <param name="strStoredProc"> ,   </param>
		/// <param name="iParamType">   </param>
		public ParamsDialog(DataTable dtParams, string strStoredProc, int iParamType) : this()
		{
			Command cmdProc = new Command();
			cmdProc.CommandName = strStoredProc;
			cmdProc.InputParams["@ParamTypeId"] = iParamType;
			cmdProc.Execute(true);
			if (cmdProc.DataSet.Tables[0].Rows.Count > 0)
				InitDialog(dtParams, cmdProc.DataSet.Tables[0], null);
		}

		/// <remarks>
		///          
		///     <see cref="ParamsDialog(System.Data.DataTable,System.String,System.String)">ParamsDialog(System.Data.DataTable,
		///     string, string) Method</see>
		/// </remarks>
		/// <summary>    </summary>
		/// <param name="dtParams">   </param>
		/// <param name="dtParamsTuning">      </param>
		public ParamsDialog(DataTable dtParams, DataTable dtParamsTuning) : this()
		{
			InitDialog(dtParams, dtParamsTuning, null);
		}
		public ParamsDialog(DataTable dtParams, DataTable dtParamsTuning, Hashtable htLookup) : this()
		{
			InitDialog(dtParams, dtParamsTuning, htLookup);
		}

		/// <summary> </summary>
		/// <param name="dtParams">    </param>
		/// <param name="dtParamsTuning"> </param>
		private void InitDialog(DataTable dtParams, DataTable dtParamsTuning, Hashtable htLookup)
		{
			string strField;
			string strCaption;
			string strDesk;
			string strStoredProc;
			string strDataKey;
			string strDataValue;
			string strList;
			string strCategory;
			string strParamType;
			string strCustomType;
			string strMask;
			string strParamStoredProc;
			string[] strLookupParams;
			int iParamTypeParam;
			object objMinValue = DBNull.Value;
			object objMaxValue = DBNull.Value;
			object objDefaultValue = DBNull.Value;
			bool bReadOnly = false;
			bool bIsNew = false;
			bool bIsMandatory = false;
			bool bListType = false;
			bool bVisible = true;
			Param parRow;
			this.Params.Clear();

			//m_strInsertStoredProc = dtParamsTuning.Rows[0]["InsertCommand"].ToString();
			//m_strUpdateStoredProc = dtParamsTuning.Rows[0]["UpdateCommand"].ToString();
			if (dtParamsTuning.Rows.Count > 0)
			{
				m_strInsertStoredProc = GetRowField(dtParamsTuning.Rows[0], "InsertCommand").ToString();
				m_strUpdateStoredProc = GetRowField(dtParamsTuning.Rows[0], "UpdateCommand").ToString();

				//if(dtParamsTuning.Rows[0]["DialogCaption"] != DBNull.Value)
				//	this.Text = dtParamsTuning.Rows[0]["DialogCaption"].ToString();
				if (GetRowField(dtParamsTuning.Rows[0], "DialogCaption") != DBNull.Value)
					this.Text = GetRowField(dtParamsTuning.Rows[0], "DialogCaption").ToString();

				if (dtParamsTuning.Rows.Count == 0 /* || dtParams.Rows.Count == 0*/)
					return;
				int iInCount = 0;
				int iInIndex = 0;
				int iCount = dtParamsTuning.Rows.Count;
				int iIndex = 0;
				while(iIndex < iCount)
				{
					DataRow drRow = dtParamsTuning.Rows[iIndex++];
					//strField = drRow["field"].ToString();
					strField = GetRowField(drRow, "field").ToString();
					//strCategory = drRow["category"].ToString();
					strCategory = GetRowField(drRow, "category").ToString();

					
					if (dtParams.Columns.Contains(strField))
					{
						strParamType = dtParams.Columns[strField].DataType.Name;
						//strCaption		= drRow["caption"].ToString();
						strCaption = GetRowField(drRow, "caption").ToString();
						//strDesk			= drRow["desk"].ToString();
						strDesk = GetRowField(drRow, "desk").ToString();
						//strStoredProc	= drRow["storedproc"].ToString();
						strStoredProc = GetRowField(drRow, "storedproc").ToString();
						strLookupParams = strStoredProc.Split('|');
						strStoredProc = strLookupParams[0];
						//strDataKey		= drRow["datakey"].ToString();
						strDataKey = GetRowField(drRow, "datakey").ToString();
						//strDataValue	= drRow["datavalue"].ToString();
						strDataValue = GetRowField(drRow, "datavalue").ToString();
						//strList			= drRow["list"].ToString();
						strList = GetRowField(drRow, "list").ToString();

						strMask = GetRowField(drRow, "mask").ToString();


						strCustomType = GetRowField(drRow, "data_type").ToString();

						if (strCustomType.Trim() != "")
							if (!Enum.IsDefined(typeof (ParamsType), strCustomType))
								strCustomType = "";
						//partDataType = (ParamsType)Enum.Parse(typeof(ParamsType), strCustomType);


						try
						{
							if (GetRowField(drRow, "min_value") /*drRow["min_value"]*/!= DBNull.Value)
							{
								objMinValue = System.Convert.ChangeType(GetRowField(drRow, "min_value") /*drRow["min_value"]*/, dtParams.Columns[strField].DataType);
								switch (dtParams.Columns[strField].DataType.ToString())
								{
									case "System.Decimal":
										NumberFormatInfo objProvider = new NumberFormatInfo();
										objProvider.CurrencyGroupSeparator = ".";
										objProvider.CurrencyDecimalSeparator = ",";
										objMinValue = System.Convert.ToDecimal(GetRowField(drRow, "min_value") /*drRow["min_value"]*/, objProvider);
										break;
								} // switch
							} // if
							else
								objMinValue = DBNull.Value;
						} // try
						catch
						{
							objMinValue = DBNull.Value;
						} // catch
						try
						{
							if (GetRowField(drRow, "max_value") /*drRow["max_value"]*/!= DBNull.Value)
							{
								objMaxValue = System.Convert.ChangeType(GetRowField(drRow, "max_value") /*drRow["max_value"]*/, dtParams.Columns[strField].DataType);
								switch (dtParams.Columns[strField].DataType.ToString())
								{
									case "System.Decimal":
										NumberFormatInfo objProvider = new NumberFormatInfo();
										objProvider.CurrencyGroupSeparator = ".";
										objProvider.CurrencyDecimalSeparator = ",";
										objMaxValue = System.Convert.ToDecimal(GetRowField(drRow, "max_value") /*drRow["max_value"]*/, objProvider);
										break;
								} // switch
							} // if
							else
								objMaxValue = DBNull.Value;
							/*
							if(objMaxValue!=DBNull.Value && objMaxValue!=DBNull.Value && strCustomType=="Int")
							{
								strList="";
								int iTop=System.Convert.ToInt32(objMaxValue);
								int iLow=System.Convert.ToInt32(objMinValue);
								for(int i=iLow; i < iTop+1;i++)
								{
									strList+=i.ToString();
									if(i!=iTop)
										strList+=strList+'|';
								}


							}
							*/


						} // try
						catch
						{
							objMaxValue = DBNull.Value;
						} // catch
						try
						{
							if (GetRowField(drRow, "defaultvalue") /*drRow["defaultvalue"]*/!= DBNull.Value)
							{
								objDefaultValue = System.Convert.ChangeType(GetRowField(drRow, "defaultvalue") /*drRow["defaultvalue"]*/, dtParams.Columns[strField].DataType);
								switch (dtParams.Columns[strField].DataType.ToString())
								{
									case "System.Decimal":
										NumberFormatInfo objProvider = new NumberFormatInfo();
										objProvider.CurrencyGroupSeparator = ".";
										objProvider.CurrencyDecimalSeparator = ",";
										objDefaultValue = System.Convert.ToDecimal(GetRowField(drRow, "defaultvalue") /*drRow["defaultvalue"]*/, objProvider);
										break;

								} // switch
								if (strCustomType == "Boolean")
									objDefaultValue = System.Convert.ToBoolean(objDefaultValue);
								if(strCustomType == "DateTime")
									objDefaultValue = System.Convert.ToDateTime(objDefaultValue);

							} // if
							else
								objDefaultValue = DBNull.Value;
						} // try
						catch
						{
							objDefaultValue = DBNull.Value;
						} // catch
						if (GetRowField(drRow, "list_type") != DBNull.Value)
							bListType = System.Convert.ToBoolean(GetRowField(drRow, "list_type") /*drRow["list_type"]*/);
						//strParamStoredProc	= drRow["storedprocparam"].ToString();
						strParamStoredProc = GetRowField(drRow, "storedprocparam").ToString();

						if (GetRowField(drRow, "paramtypeparam") /*drRow["paramtypeparam"]*/== DBNull.Value)
							iParamTypeParam = 0;
						else
							iParamTypeParam = (int) GetRowField(drRow, "paramtypeparam") /*drRow["paramtypeparam"]*/;

						if (GetRowField(drRow, "readonly") != DBNull.Value)
							bReadOnly = System.Convert.ToBoolean(GetRowField(drRow, "readonly") /*drRow["readonly"]*/);
						if (GetRowField(drRow, "visible") != DBNull.Value)
							bVisible = System.Convert.ToBoolean(GetRowField(drRow, "visible") /*drRow["readonly"]*/);
						if (GetRowField(drRow, "isnew") != DBNull.Value)
							bIsNew = System.Convert.ToBoolean(GetRowField(drRow, "isnew") /*drRow["isnew"]*/);
						if (GetRowField(drRow, "must_be_filled") != DBNull.Value)
							bIsMandatory = System.Convert.ToBoolean(GetRowField(drRow, "must_be_filled") /*drRow["isnew"]*/);

						if (strList.Trim() != "")
						{
							if (dtParams.Rows.Count > 0 && dtParams.Rows[0][strField] is string)
								parRow = new Param(strField, strCaption, ParamsType.List, strCategory, strDesk);
							else
								parRow = new Param(strField, strCaption, ParamsType.List, strCategory, strDesk, dtParams.Rows.Count > 0 ? System.Convert.ToInt32(dtParams.Rows[0][strField]) : 0);
							parRow.List = new ArrayList();
							parRow.ListType = bListType;

							string[] strSplit = strList.Split(new char[] {'|'});
							iInCount = strSplit.Length;
							iInIndex = 0;
							while(iInIndex < iInCount)
							{
								string strItem = strSplit[iInIndex++];
								parRow.List.Add(strItem);
							}
							if (dtParams.Rows.Count > 0 && dtParams.Rows[0][strField] is string)
							{
								int iListIndex = parRow.List.IndexOf(dtParams.Rows[0][strField]);
								if (iListIndex < 0)
									iListIndex = 0;
								parRow.DefaultValue = iListIndex;
							} // if
						} // if
						else if (strStoredProc.Trim() != "" && strDataKey.Trim() != "" && strDataValue.Trim() != "")
						{
							try
							{
								if(objDefaultValue != DBNull.Value && objMinValue != DBNull.Value)
								{
									if(System.Convert.ToInt32(objDefaultValue) < System.Convert.ToInt32(objMinValue))
										objDefaultValue = DBNull.Value;
								}

							}
							catch(Exception excLookup)
							{
								//MessageBox.Show(Softel.Windows.Forms.ResourceControl.String32, ResourceControl.Error1, MessageBoxButtons.OK, MessageBoxIcon.Error, excLookup.ToString());
							}
	
									DataTable dtLookup;
							Command cmdProc = new Command();
							cmdProc.CommandName = strStoredProc;
							if(strLookupParams.Length > 1 )
							//if(htLookup != null && strLookupParams.Length > 1 && htLookup.Count > 0)
							{
								int iIndexLookup = 1;
								int iCountLookup = strLookupParams.Length;
								string strKey;
								while(iIndexLookup < iCountLookup)
								{
									strKey = strLookupParams[iIndexLookup++];
									if(htLookup != null && htLookup.Count > 0 && htLookup.ContainsKey(strKey))
										cmdProc.InputParams["@" + strKey] = htLookup[strKey];
									else
										if(dtParams.Columns.Contains(strKey) && dtParams.Rows.Count >0)
											cmdProc.InputParams["@" + strKey] = dtParams.Rows[0][strKey];
											
											
								}
							}
							cmdProc.Execute(true);
							dtLookup = cmdProc.DataSet.Tables[0];
							if (dtParams.Rows.Count > 0)
								parRow = new Param(strField, strCaption, ParamsType.Data, strCategory, strDesk, dtParams.Rows[0][strField]);
							else
								parRow = new Param(strField, strCaption, ParamsType.Data, strCategory, strDesk, objDefaultValue);
							parRow.Table = dtLookup;
							try
							{
								parRow.Table.PrimaryKey = new DataColumn[] {parRow.Table.Columns[strDataKey]};
								parRow.StoredProc = strStoredProc;
								parRow.DataValue = strDataValue;
								parRow.DataKey = strDataKey;
							}
							catch(Exception excLookup)
							{
								MessageBox.Show(Softel.Windows.Forms.ResourceControl.String32, ResourceControl.Error1, MessageBoxButtons.OK, MessageBoxIcon.Error, excLookup.ToString());
							}
							parRow.IsNew = bIsNew;
							parRow.InsertStoredProc = strParamStoredProc;
							parRow.InsertParamType = iParamTypeParam;
						} // if
						else
						{
							if (dtParams.Rows.Count > 0 && strCustomType != "CustomForm")
							{
								if (strCustomType.Trim() == "")
									parRow = new Param(strField, strCaption, "System." + strParamType, strCategory, strDesk, dtParams.Rows[0][strField]==DBNull.Value?objDefaultValue:dtParams.Rows[0][strField]);
								else
									parRow = new Param(strField, strCaption, (ParamsType) Enum.Parse(typeof (ParamsType), strCustomType), strCategory, strDesk,dtParams.Rows[0][strField]==DBNull.Value?objDefaultValue:dtParams.Rows[0][strField] );
							} // if
							else
							{
								if (strCustomType.Trim() == "")
									parRow = new Param(strField, strCaption, "System." + strParamType, strCategory, strDesk, objDefaultValue);
								else
									parRow = new Param(strField, strCaption, (ParamsType) Enum.Parse(typeof (ParamsType), strCustomType), strCategory, strDesk, objDefaultValue);
							} // else

						} // else
						try
						{
							if (strCustomType == "CustomForm" && GetRowField(drRow, "CustomForm") != DBNull.Value)
							{
								string strAssemblyName = GetRowField(drRow, "CustomForm").ToString();
								int iInd1 = strAssemblyName.LastIndexOf(".");
								string strMethod = strAssemblyName.Substring(iInd1 + 1);
								FbConnection fbConnection = (FbConnection) Thread.GetData(Thread.GetNamedDataSlot("FbConnection"));
								Assembly asCustomForm = LoadAssembly(fbConnection, strAssemblyName, "iv");
								Type[] typeCustom = asCustomForm.GetTypes();
								iInCount = typeCustom.Length;
								iInIndex = 0;
								while(iInIndex < iInCount)
								{
									Type t = typeCustom[iInIndex++];
									string strCurMethod = t.ToString();
									int iInd2 = strCurMethod.LastIndexOf(".");
									string strTempMethod = strCurMethod.Substring(iInd2 + 1);
									if (strTempMethod == strMethod)
									{
										parRow.CustomForm = (ICustomForm) t.InvokeMember(strMethod, BindingFlags.CreateInstance, null, null, null);
										//((ICustomValue)(parRow)).Caption=objDefaultValue.ToString();

										break;
									}
								}
							}
						}
						catch
						{
						}


						parRow.IsMandatory = bIsMandatory;
						parRow.MinValue = objMinValue == DBNull.Value ? null : objMinValue;
						parRow.MaxValue = objMaxValue == DBNull.Value ? null : objMaxValue;
						parRow.ReadOnly = bReadOnly;
						parRow.Visible = bVisible;
						parRow.Mask = strMask;
						parRow.InsertField = GetRowField(drRow, "ins_proc_param") /*drRow["ins_proc_param"]*/.ToString();
						if (parRow.InsertField == "")
							parRow.InsertField = null;
						parRow.UpdateField = GetRowField(drRow, "upd_proc_param") /*drRow["upd_proc_param"]*/.ToString();
						if (parRow.UpdateField == "")
							parRow.UpdateField = null;
						parRow.OnNewItem += new EventHandler(ParamOnNewItem);
						this.Params.Add(parRow);

					} // if

				} // foreach  (drRow)
			} // if
		}

		/// <summary>    ,  </summary>
		/// <returns>  </returns>
		/// <param name="drRow"> </param>
		/// <param name="sFieldName"> </param>
		private object GetRowField(DataRow drRow, string sFieldName)
		{
			object obj = DBNull.Value;
			if (drRow.Table.Columns.Contains(sFieldName))
				obj = drRow[sFieldName];

			return obj;
		}

		/// <summary>    </summary>
		/// <value>     <see cref="ParamBag"/></value>
		public ParamBag ParamTable
		{
			get { return m_objParamBag; }
		}

		public ParamsDialogCollection ParamsDialogs
		{
			set
			{
				
				for (int i = 0; i < Params.Count; i++)
				{
					if (Params[i].Type == ParamsType.CustomForm)
						Params[i].CustomForm.ParamsDialogs = value;

				}

			}
		}

		/// <summary>
		///  
		/// </summary>
		/// <param name="disposing"> <see cref="Form.Dispose"/></param>
		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		#region Windows Form Designer generated code

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(ParamsDialog));
			this.m_objPropertyGrid = new System.Windows.Forms.PropertyGrid();
			this.m_btnOk = new Softel.Windows.Forms.ButtonXP();
			this.m_btnCancel = new Softel.Windows.Forms.ButtonXP();
			this.SuspendLayout();
			// 
			// m_objPropertyGrid
			// 
			this.m_objPropertyGrid.AccessibleDescription = resources.GetString("m_objPropertyGrid.AccessibleDescription");
			this.m_objPropertyGrid.AccessibleName = resources.GetString("m_objPropertyGrid.AccessibleName");
			this.m_objPropertyGrid.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("m_objPropertyGrid.Anchor")));
			this.m_objPropertyGrid.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("m_objPropertyGrid.BackgroundImage")));
			this.m_objPropertyGrid.CommandsVisibleIfAvailable = true;
			this.m_objPropertyGrid.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("m_objPropertyGrid.Dock")));
			this.m_objPropertyGrid.Enabled = ((bool)(resources.GetObject("m_objPropertyGrid.Enabled")));
			this.m_objPropertyGrid.Font = ((System.Drawing.Font)(resources.GetObject("m_objPropertyGrid.Font")));
			this.m_objPropertyGrid.HelpVisible = ((bool)(resources.GetObject("m_objPropertyGrid.HelpVisible")));
			this.m_objPropertyGrid.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("m_objPropertyGrid.ImeMode")));
			this.m_objPropertyGrid.LargeButtons = false;
			this.m_objPropertyGrid.LineColor = System.Drawing.SystemColors.ScrollBar;
			this.m_objPropertyGrid.Location = ((System.Drawing.Point)(resources.GetObject("m_objPropertyGrid.Location")));
			this.m_objPropertyGrid.Name = "m_objPropertyGrid";
			this.m_objPropertyGrid.PropertySort = System.Windows.Forms.PropertySort.Alphabetical;
			this.m_objPropertyGrid.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("m_objPropertyGrid.RightToLeft")));
			this.m_objPropertyGrid.Size = ((System.Drawing.Size)(resources.GetObject("m_objPropertyGrid.Size")));
			this.m_objPropertyGrid.TabIndex = ((int)(resources.GetObject("m_objPropertyGrid.TabIndex")));
			this.m_objPropertyGrid.Text = resources.GetString("m_objPropertyGrid.Text");
			this.m_objPropertyGrid.ViewBackColor = System.Drawing.SystemColors.Window;
			this.m_objPropertyGrid.ViewForeColor = System.Drawing.SystemColors.WindowText;
			this.m_objPropertyGrid.Visible = ((bool)(resources.GetObject("m_objPropertyGrid.Visible")));
			this.m_objPropertyGrid.PropertyValueChanged += new System.Windows.Forms.PropertyValueChangedEventHandler(this.m_objPropertyGrid_PropertyValueChanged);
			// 
			// m_btnOk
			// 
			this.m_btnOk.AccessibleDescription = resources.GetString("m_btnOk.AccessibleDescription");
			this.m_btnOk.AccessibleName = resources.GetString("m_btnOk.AccessibleName");
			this.m_btnOk.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("m_btnOk.Anchor")));
			this.m_btnOk.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(0)), ((System.Byte)(212)), ((System.Byte)(208)), ((System.Byte)(200)));
			this.m_btnOk.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("m_btnOk.BackgroundImage")));
			this.m_btnOk.DefaultScheme = true;
			this.m_btnOk.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.m_btnOk.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("m_btnOk.Dock")));
			this.m_btnOk.Enabled = ((bool)(resources.GetObject("m_btnOk.Enabled")));
			this.m_btnOk.Font = ((System.Drawing.Font)(resources.GetObject("m_btnOk.Font")));
			this.m_btnOk.Hint = "";
			this.m_btnOk.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("m_btnOk.ImeMode")));
			this.m_btnOk.Location = ((System.Drawing.Point)(resources.GetObject("m_btnOk.Location")));
			this.m_btnOk.Name = "m_btnOk";
			this.m_btnOk.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("m_btnOk.RightToLeft")));
			this.m_btnOk.Scheme = Softel.Windows.Forms.ButtonXP.Schemes.Blue;
			this.m_btnOk.Size = ((System.Drawing.Size)(resources.GetObject("m_btnOk.Size")));
			this.m_btnOk.TabIndex = ((int)(resources.GetObject("m_btnOk.TabIndex")));
			this.m_btnOk.Text = resources.GetString("m_btnOk.Text");
			this.m_btnOk.Visible = ((bool)(resources.GetObject("m_btnOk.Visible")));
			// 
			// m_btnCancel
			// 
			this.m_btnCancel.AccessibleDescription = resources.GetString("m_btnCancel.AccessibleDescription");
			this.m_btnCancel.AccessibleName = resources.GetString("m_btnCancel.AccessibleName");
			this.m_btnCancel.Anchor = ((System.Windows.Forms.AnchorStyles)(resources.GetObject("m_btnCancel.Anchor")));
			this.m_btnCancel.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(0)), ((System.Byte)(212)), ((System.Byte)(208)), ((System.Byte)(200)));
			this.m_btnCancel.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("m_btnCancel.BackgroundImage")));
			this.m_btnCancel.DefaultScheme = true;
			this.m_btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.m_btnCancel.Dock = ((System.Windows.Forms.DockStyle)(resources.GetObject("m_btnCancel.Dock")));
			this.m_btnCancel.Enabled = ((bool)(resources.GetObject("m_btnCancel.Enabled")));
			this.m_btnCancel.Font = ((System.Drawing.Font)(resources.GetObject("m_btnCancel.Font")));
			this.m_btnCancel.Hint = "";
			this.m_btnCancel.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("m_btnCancel.ImeMode")));
			this.m_btnCancel.Location = ((System.Drawing.Point)(resources.GetObject("m_btnCancel.Location")));
			this.m_btnCancel.Name = "m_btnCancel";
			this.m_btnCancel.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("m_btnCancel.RightToLeft")));
			this.m_btnCancel.Scheme = Softel.Windows.Forms.ButtonXP.Schemes.Blue;
			this.m_btnCancel.Size = ((System.Drawing.Size)(resources.GetObject("m_btnCancel.Size")));
			this.m_btnCancel.TabIndex = ((int)(resources.GetObject("m_btnCancel.TabIndex")));
			this.m_btnCancel.Text = resources.GetString("m_btnCancel.Text");
			this.m_btnCancel.Visible = ((bool)(resources.GetObject("m_btnCancel.Visible")));
			// 
			// ParamsDialog
			// 
			this.AcceptButton = this.m_btnOk;
			this.AccessibleDescription = resources.GetString("$this.AccessibleDescription");
			this.AccessibleName = resources.GetString("$this.AccessibleName");
			this.AutoScaleBaseSize = ((System.Drawing.Size)(resources.GetObject("$this.AutoScaleBaseSize")));
			this.AutoScroll = ((bool)(resources.GetObject("$this.AutoScroll")));
			this.AutoScrollMargin = ((System.Drawing.Size)(resources.GetObject("$this.AutoScrollMargin")));
			this.AutoScrollMinSize = ((System.Drawing.Size)(resources.GetObject("$this.AutoScrollMinSize")));
			this.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("$this.BackgroundImage")));
			this.CancelButton = this.m_btnCancel;
			this.ClientSize = ((System.Drawing.Size)(resources.GetObject("$this.ClientSize")));
			this.Controls.Add(this.m_btnCancel);
			this.Controls.Add(this.m_btnOk);
			this.Controls.Add(this.m_objPropertyGrid);
			this.Enabled = ((bool)(resources.GetObject("$this.Enabled")));
			this.Font = ((System.Drawing.Font)(resources.GetObject("$this.Font")));
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.ImeMode = ((System.Windows.Forms.ImeMode)(resources.GetObject("$this.ImeMode")));
			this.Location = ((System.Drawing.Point)(resources.GetObject("$this.Location")));
			this.MaximizeBox = false;
			this.MaximumSize = ((System.Drawing.Size)(resources.GetObject("$this.MaximumSize")));
			this.MinimizeBox = false;
			this.MinimumSize = ((System.Drawing.Size)(resources.GetObject("$this.MinimumSize")));
			this.Name = "ParamsDialog";
			this.RightToLeft = ((System.Windows.Forms.RightToLeft)(resources.GetObject("$this.RightToLeft")));
			this.ShowInTaskbar = false;
			this.StartPosition = ((System.Windows.Forms.FormStartPosition)(resources.GetObject("$this.StartPosition")));
			this.Text = resources.GetString("$this.Text");
			this.Closing += new System.ComponentModel.CancelEventHandler(this.ParamsDialog_Closing);
			this.Load += new System.EventHandler(this.ParamsDialog_Load);
			this.ResumeLayout(false);

		}

		#endregion

		/// <summary>
		///     
		/// </summary>
		/// <value>     <see cref="ParamCollection"/></value>
		public ParamCollection Params
		{
			get { return m_objParamCollection; }
		}

		/// <summary>  <see cref="ParamsDialog"/>   </summary>
		/// <value>  <see cref="PropertyGrid"/></value>
		/// <example>
		///       <see cref="ParamsDialog"/>   
		///     <code lang="CS" title=" :">
		/// ParamsDialog pdQuarter = new ParamsDialog(cmdQuarter.DataSet.Tables[0], cmdParams.DataSet.Tables[0]);
		/// pdQuarter.OnAfterSetValue += new ParamsDialogEventHandler(pdQuarter_OnSetValue);
		/// ContextMenu cmQuarter = new ContextMenu();
		/// cmQuarter.Popup += new EventHandler(cmQuarter_Popup);
		/// pdQuarter.PropertyGrid.ContextMenu = cmQuarter;
		/// pdQuarter.Toolbar = false;
		/// pdQuarter.PropertySort = PropertySort.Categorized;
		/// pdQuarter.PropertyGrid.Dock = DockStyle.Fill;
		/// Quarter frmQuarter = new Quarter(((NodeTag)e.Node.Tag).m_iId);
		/// using(Command cmdTemplate = new Command())
		/// {
		///     cmdTemplate.CommandName = "ksp_getQuarterTemplate";
		///     cmdTemplate.InputParams["@Quarter_id"] = ((NodeTag)e.Node.Tag).m_iId;
		///     cmdTemplate.Execute(true);
		///     if(cmdTemplate.DataSet.Tables[0].Rows.Count &gt; 0)
		///         frmQuarter.RelationField = cmdTemplate.DataSet.Tables[0].Rows[0]["Relation_field"].ToString();
		/// }
		/// frmQuarter.OnChange += new EventHandler(frmQuarter_OnChange);
		/// frmQuarter.Dock = DockStyle.Fill;
		/// frmQuarter.Outlay = pdQuarter.PropertyGrid;
		/// frmQuarter.Parent = m_pnlDetail;
		///     </code>
		/// </example>
		public PropertyGrid PropertyGrid
		{
			get { return m_objPropertyGrid; }
		}

		/// <summary>
		/// ,         
		/// </summary>
		/// <param name="sender">  </param>
		/// <param name="eaRealValue"> </param>
		private void m_objParamBag_GetValue(object sender, ParamEventArgs eaRealValue)
		{
			ParamEventArgs eaValue = eaRealValue;
			if (OnGetValue != null)
				eaValue = OnGetValue(this, eaValue);
			switch (eaValue.Property.Type)
			{
				case ParamsType.List:
					eaValue.Index = (int) m_htValue[eaValue.Property.Name];
					eaValue.Value = eaValue.Property.List[eaValue.Index];
					break;
				case ParamsType.Data:
					eaValue.DataKey = m_htValue[eaValue.Property.Name];
					if (eaValue.DataKey != DBNull.Value)
					{
						DataRow drFind = eaValue.Property.Table.Rows.Find(eaValue.DataKey);
						if (drFind != null)
							eaValue.Value = drFind[eaValue.Property.DataValue];
					}
					break;
				default:
					eaValue.Value = m_htValue[eaValue.Property.Name];
					break;
			}
		}

		/// <summary>
		///       
		/// 
		/// </summary>
		/// <param name="eaRealValue">     </param>
		public void SetValue(ParamEventArgs eaRealValue)
		{
			m_objParamBag_SetValue(this, eaRealValue);
			m_objPropertyGrid.SelectedObject = m_objParamBag;
		}

		/// <summary>
		/// ,     
		/// </summary>
		/// <param name="sender">  </param>
		/// <param name="eaRealValue"> </param>
		private void m_objParamBag_SetValue(object sender, ParamEventArgs eaRealValue)
		{
			ParamEventArgs eaValue = eaRealValue;
			if (OnSetValue != null && !eaRealValue.Equals(m_parOnce))
				eaValue = OnSetValue(this, eaValue);
			switch (eaValue.Property.Type)
			{
				case ParamsType.List:
					m_htValue[eaValue.Property.Name] = eaValue.Index;
					if (!eaValue.Property.ListType)
					{
						if (eaValue.Property.InsertField != null)
							m_htInsertValue[eaValue.Property.InsertField] = eaValue.Index;
						if (eaValue.Property.UpdateField != null)
							m_htUpdateValue[eaValue.Property.UpdateField] = eaValue.Index;
					}
					else
					{
						if (eaValue.Property.InsertField != null)
							m_htInsertValue[eaValue.Property.InsertField] = eaValue.Property.List[eaValue.Index];
						if (eaValue.Property.UpdateField != null)
							m_htUpdateValue[eaValue.Property.UpdateField] = eaValue.Property.List[eaValue.Index];
					}
					break;
				case ParamsType.Data:
					m_htValue[eaValue.Property.Name] = eaValue.DataKey;
					if (eaValue.Property.InsertField != null)
						m_htInsertValue[eaValue.Property.InsertField] = eaValue.DataKey;
					if (eaValue.Property.UpdateField != null)
						m_htUpdateValue[eaValue.Property.UpdateField] = eaValue.DataKey;
					break;
				default:
					m_htValue[eaValue.Property.Name] = eaValue.Value;
					if (eaValue.Property.InsertField != null)
						m_htInsertValue[eaValue.Property.InsertField] = eaValue.Value;
					if (eaValue.Property.UpdateField != null)
						m_htUpdateValue[eaValue.Property.UpdateField] = eaValue.Value;
					break;
			}
			if (OnAfterSetValue != null && !eaRealValue.Equals(m_parOnce))
				OnAfterSetValue(this, eaValue);
			m_parOnce = eaRealValue;
		}

		/// <summary>
		/// ,       , 
		///   
		/// </summary>
		/// <value>
		/// 	<list type="bullet">
		/// 		<item>True -     </item>
		/// 		<item>False -    </item>
		/// 	</list>
		/// </value>
		public bool RestoreDefault
		{
			get { return m_bRestoreDefault; }
			set { m_bRestoreDefault = value; }
		}

		/// <summary>,     </summary>
		/// <param name="sender">  </param>
		/// <param name="e"> </param>
		private void ParamsDialog_Load(object sender, EventArgs e)
		{
			if (m_bRestoreDefault)
				SetEvents();
		}

		/// <summary>
		///  ParamsDialog
		/// </summary>
		public override void Refresh()
		{
			SetEvents();
		}

		/// <summary>    </summary>
		private void SetEvents()
		{
			m_objParamBag.Params = m_objParamCollection;
			Param[] objParams = m_objParamCollection.ToArray();
			int iCount = objParams.Length;
			int iIndex = 0;
			while(iIndex < iCount)
			{
				Param objParam = objParams[iIndex++];
				m_htValue[objParam.Name] = objParam.DefaultValue;
				if (objParam.InsertField != null)
					m_htInsertValue[objParam.InsertField] = objParam.DefaultValue;
				if (objParam.UpdateField != null)
					m_htUpdateValue[objParam.UpdateField] = objParam.DefaultValue;
			}
			m_objParamBag.GetValue += new ParamEventHandler(m_objParamBag_GetValue);
			m_objParamBag.SetValue += new ParamEventHandler(m_objParamBag_SetValue);
			m_objPropertyGrid.SelectedObject = m_objParamBag;
		}

		/// <summary>
		///  
		/// </summary>
		public Hashtable Value
		{
			get { return m_htValue; }
		}

		/// <summary>     </summary>
		/// <value>     <see cref="Hashtable"/></value>
		public Hashtable InsertValue
		{
			get { return m_htInsertValue; }
		}

		/// <summary>     </summary>
		/// <value>     <see cref="Hashtable"/></value>
		public Hashtable UpdateTable
		{
			get { return m_htUpdateValue; }
		}

		/// <summary>     </summary>
		/// <value>  <see cref="string"/></value>
		public string InsertStoredProc
		{
			get { return m_strInsertStoredProc; }
		}

		/// <summary>     </summary>
		/// <value>     <see cref="Hashtable"/></value>
		public string UpdateStoredProc
		{
			get { return m_strUpdateStoredProc; }
		}

		/// <summary>
		///     
		/// </summary>
		/// <value>     <see cref="PropertySort"/></value>
		public PropertySort Sort
		{
			get { return m_objPropertyGrid.PropertySort; }
			set { m_objPropertyGrid.PropertySort = value; }
		}

		/// <summary>,        </summary>
		/// <param name="sender">,  </param>
		/// <param name="e"> </param>
		private void ParamOnNewItem(object sender, EventArgs e)
		{
			Param parSender = (Param) sender;

			parSender.Table.Rows.Clear();
			ParamsDialog pdNewItem = new ParamsDialog(parSender.Table, parSender.InsertStoredProc, parSender.InsertParamType);
			if (pdNewItem.ShowDialog() == DialogResult.OK)
			{
				Command cmdNew = new Command();
				cmdNew.CommandName = pdNewItem.InsertStoredProc;
				cmdNew.InputParams = pdNewItem.InsertValue;
				cmdNew.Execute(true);
			}

			Command cmdProc = new Command();
			cmdProc.CommandName = parSender.StoredProc;
			cmdProc.Execute(true);
			parSender.Table = cmdProc.DataSet.Tables[0];
			parSender.Table.PrimaryKey = new DataColumn[] {parSender.Table.Columns[parSender.DataKey]};
		}

		/// <summary>,     </summary>
		/// <param name="s">  </param>
		/// <param name="e"> </param>
		private void m_objPropertyGrid_PropertyValueChanged(object s, PropertyValueChangedEventArgs e)
		{
		}

		/// <summary>
		///     ,     
		///     <see cref="PropertyGrid"/>
		/// </summary>
		/// <value>
		/// 	<list type="bullet">
		/// 		<item>True -    </item>
		/// 		<item>False -     </item>
		/// 	</list>
		/// </value>
		public bool Toolbar
		{
			get { return m_objPropertyGrid.ToolbarVisible; }
			set { m_objPropertyGrid.ToolbarVisible = value; }
		}

		/// <summary>    </summary>
		/// <value>     <see cref="PropertySort"/></value>
		public PropertySort PropertySort
		{
			get { return m_objPropertyGrid.PropertySort; }
			set { m_objPropertyGrid.PropertySort = value; }
		}

		/// <summary>,     <see cref="PropertyGrid"/></summary>
		/// <param name="sender">  </param>
		/// <param name="e"> </param>
		private void m_objPropertyGrid_ParentChanged(object sender, EventArgs e)
		{
			if (m_bRestoreDefault)
				SetEvents();
		}

		/// <summary>,    </summary>
		/// <param name="sender">  </param>
		/// <param name="e">
		/// 	<para> </para>
		/// 	<para>e.Cancel -   True,     
		///       ,  - False</para>
		/// </param>
		private void ParamsDialog_Closing(object sender, CancelEventArgs e)
		{
			bool bSuccess = true;
			if (DialogResult == DialogResult.OK)
			{
				foreach (DictionaryEntry deCur in this.Value)
				{
					if ((deCur.Value == DBNull.Value || deCur.Value == null || (deCur.Value is string && ((string) deCur.Value) == "")) && this.Params[deCur.Key.ToString()].IsMandatory)
					{
						MessageBox.Show(string.Format(Softel.Windows.Forms.ResourceControl.String33, this.Params[deCur.Key.ToString()].Caption), Softel.Windows.Forms.ResourceControl.String34);
						bSuccess = false;
//						this.RestoreDefault = false;
						break;
					}
				}
			}
			e.Cancel = !bSuccess;
		}

		/// <summary>
///  Assembly    CustomForm
/// </summary>
/// <param name="fbConnection">c</param>
/// <param name="strAsmName"> </param>
/// <param name="culture"></param>
/// <returns></returns>
		public Assembly LoadAssembly(FbConnection fbConnection, string strAsmName, string culture)
		{
			Assembly objAssembly = null;
			objAssembly = Assembly.Load(strAsmName);
			if(objAssembly ==null)
			{
				Assembly[] asmsDomain = AppDomain.CurrentDomain.GetAssemblies();
				int iCount = asmsDomain.Length;
				int iIndex = 0;
				while(iIndex < iCount)
				{
					Assembly objCurrentAssembly = asmsDomain[iIndex++];
					if (strAsmName == objCurrentAssembly.FullName)
					{
						objAssembly = objCurrentAssembly;
						break;
					}
				}
			}
			if (objAssembly == null)
			{
				SSP sspAsm = new SSP();
				sspAsm.DatabaseType = AdoDbType.FireBird;
				sspAsm.Connection = fbConnection;
				sspAsm.ParamsRange["@ASMNAME"] = strAsmName;
				sspAsm.ParamsRange["@CULTURE"] = culture;
				sspAsm.ParamsRange["@VERSION"] = null;
				sspAsm.Execute("asm_getAsm");
				if (sspAsm.DataSet != null && sspAsm.DataSet.Tables.Count > 0 && sspAsm.DataSet.Tables[0].Rows.Count > 0)
				{
					if (sspAsm.DataSet.Tables[0].Rows[0]["Data"] != DBNull.Value)
						objAssembly = AppDomain.CurrentDomain.Load((byte[]) sspAsm.DataSet.Tables[0].Rows[0]["Data"]);
				}
			}
			return objAssembly;
		}

	}
	/// <summary>     <see cref="ParamsDialog"/></summary>
	/// <param name="pdSender">- </param>
	/// <param name="parValue"> </param>
	public delegate ParamEventArgs ParamsDialogEventHandler(ParamsDialog pdSender, ParamEventArgs parValue);
}
/********************* SniipSoftel *****************
 *
 *    $History: ParamsDialog.cs $
 *   
 *   *****************  Version 61  *****************
 *   User: Gavrilov     Date: 21.05.07   Time: 17:52
 *   Updated in $/Net/Common/ClientControl/ParamsDialog
 *   
 *   *****************  Version 60  *****************
 *   User: Serg_ch      Date: 16.05.07   Time: 16:31
 *   Updated in $/Net/Common/ClientControl/ParamsDialog
 *   
 *   *****************  Version 59  *****************
 *   User: Gavrilov     Date: 27.03.07   Time: 19:26
 *   Updated in $/Net/Common/ClientControl/ParamsDialog
 *   
 *   *****************  Version 58  *****************
 *   User: Serg_ch      Date: 26.03.07   Time: 20:42
 *   Updated in $/Net/Common/ClientControl/ParamsDialog
 *   
 *   *****************  Version 56  *****************
 *   User: Serg_ch      Date: 27.02.07   Time: 13:39
 *   Updated in $/Net/Common/SoftelControl/ParamsDialog
 *   
 *   *****************  Version 55  *****************
 *   User: Gavrilov     Date: 5.02.07    Time: 15:53
 *   Updated in $/Net/Common/SoftelControl/ParamsDialog
 *   
 *   *****************  Version 54  *****************
 *   User: Serg_ch      Date: 1.02.07    Time: 12:23
 *   Updated in $/Net/Common/SoftelControl/ParamsDialog
 *   
 *   *****************  Version 53  *****************
 *   User: Serg_ch      Date: 30.01.07   Time: 19:09
 *   Updated in $/Net/Common/SoftelControl/ParamsDialog
 *   
 *   *****************  Version 52  *****************
 *   User: Serg_ch      Date: 29.01.07   Time: 17:45
 *   Updated in $/Net/Common/SoftelControl/ParamsDialog
 *   
 *   *****************  Version 51  *****************
 *   User: Serg_ch      Date: 29.01.07   Time: 17:17
 *   Updated in $/Net/Common/SoftelControl/ParamsDialog
 *   
 *   *****************  Version 50  *****************
 *   User: Serg_ch      Date: 29.01.07   Time: 16:08
 *   Updated in $/Net/Common/SoftelControl/ParamsDialog
 *   
 *   *****************  Version 49  *****************
 *   User: Serg_ch      Date: 29.01.07   Time: 15:02
 *   Updated in $/Net/Common/SoftelControl/ParamsDialog
 *   
 *   *****************  Version 48  *****************
 *   User: Serg_ch      Date: 29.01.07   Time: 14:52
 *   Updated in $/Net/Common/SoftelControl/ParamsDialog
 *   
 *   *****************  Version 47  *****************
 *   User: Serg_ch      Date: 24.01.07   Time: 20:16
 *   Updated in $/Net/Common/SoftelControl/ParamsDialog
 *   
 *   *****************  Version 46  *****************
 *   User: Serg_ch      Date: 24.01.07   Time: 14:46
 *   Updated in $/Net/Common/SoftelControl/ParamsDialog
 *   
 *   *****************  Version 45  *****************
 *   User: Autobuild    Date: 19.12.06   Time: 23:18
 *   Updated in $/Net/Common/SoftelControl/PARAMSDIALOG
 *
 ********************* SniipSoftel *****************/
