/********************* SniipSoftel *****************
 *
 *   $Author: Autobuild $
 *   $Revision: 4 $
 *   $Date: 19.12.06 23:18 $
 *
 ********************* SniipSoftel *****************/
/*
 *   Copyright (c) 2001 Softel
 *
 *   $History: Attributes.cs $
 *   
 *   *****************  Version 4  *****************
 *   User: Autobuild    Date: 19.12.06   Time: 23:18
 *   Updated in $/Net/Common/SoftelControl/PROPERTYGRID
 * 
 **************************************************************************************************/
using System;
using System.ComponentModel;
using System.Reflection;
using System.Resources;
using System.Threading;

namespace Softel.Windows.Forms
{
	// This attribute may be added to properties to control the resource table name and 
	// keys for accesging the property name and description. 
	[AttributeUsage(AttributeTargets.Property, AllowMultiple=false, Inherited=true)]
	public class LocalizedPropertyAttribute : Attribute
	{
		private String resourceName = "";
		private String resourceTable = "";
		private String resourceDescription = "";

		public LocalizedPropertyAttribute(String name)
		{
			resourceName = name;
		}

		// The key for a property name into the resource table.
		// If empty the property name will be used by default.
		public String Name
		{
			get { return resourceName; }
			set { resourceName = value; }
		}

		// The key for a property description into the resource table.
		// If empty the property name appended by 'Description' will be used by default.
		public String Description
		{
			get { return resourceDescription; }
			set { resourceDescription = value; }
		}

		// The name fully qualified name of the resource table to use, f.ex.
		// "LocalizedPropertyGrid.Person".
		public String Table
		{
			get { return resourceTable; }
			set { resourceTable = value; }
		}
	}

	// This attribute may be added to properties to control the resource table name and 
	// keys for accesging the property name and description. 
	[AttributeUsage(AttributeTargets.Class, AllowMultiple=false, Inherited=true)]
	public class LocalizedObjectAttribute : Attribute
	{
		private String resourceTable = "";

		public LocalizedObjectAttribute(String name)
		{
			resourceTable = name;
		}

		// resource table name to use for the class. 
		// If this string is empty the name of the class will be used by default. 
		public String Table
		{
			get { return resourceTable; }
		}
	}

	// This attribute class extends the CategoryAttribute class from the .NET framework
	// to support localization.
	[AttributeUsage(AttributeTargets.All, AllowMultiple=false, Inherited=true)]
	public class LocalizedCategoryAttribute : CategoryAttribute
	{
		private Assembly resourceAssembly;
		private String resourceTable = "";

		public LocalizedCategoryAttribute() : base()
		{
		}

		public LocalizedCategoryAttribute(string name) :
			base(name)
		{
		}

		// The name fully qualified name of the resource table to use, f.ex.
		// "LocalizedPropertyGrid.Person".
		public string TableName
		{
			set { resourceTable = value; }
			get { return resourceTable; }
		}

		public Assembly Assembly
		{
			get { return resourceAssembly; }
			set { resourceAssembly = value; }
		}

		// This method will be called by the component framework to get a localized
		// string. Note: Only called once on first access.
		protected override string GetLocalizedString(string value)
		{
			string baseStr = base.GetLocalizedString(value);

			// Now use table name and display name id to access the resources.  
			while (true)
			{
				ResourceManager rm = new ResourceManager(resourceTable, resourceAssembly /*GetExecutingAssembly()*/);

				// Get the string from the resources.
				try
				{
					return rm.GetString(value, Thread.CurrentThread.CurrentCulture);
				}
				catch (Exception)
				{
//					System.Collections.IEnumerator enumAss = Assembly.GetEntryAssembly().GetReferencedAssemblies().GetEnumerator();
//					while(enumAss.MoveNext())
//					{
//						try
//						{
////							Assembly.GetAssembly(PropertyGrid);
//							rm = new ResourceManager(resourceTable, Assembly.GetAssembly(System.Windows.Forms.PropertyGrid));
//							string _strResource = rm.GetString(value);
//						}
//						catch(Exception)
//						{}
//					}

					return value;
				}
			}
		}
	}

	[AttributeUsage(AttributeTargets.All, AllowMultiple=false, Inherited=true)]
	public class VisibleAttribute : Attribute
	{
		public VisibleAttribute(bool bVisible)
		{
			m_bVisible = bVisible;
		}

		private bool m_bVisible = true;

		public bool Visible
		{
			get { return m_bVisible; }
			set { m_bVisible = value; }
		}
	}
}
/********************* SniipSoftel *****************
 *
 *    $History: Attributes.cs $
 *   
 *   *****************  Version 4  *****************
 *   User: Autobuild    Date: 19.12.06   Time: 23:18
 *   Updated in $/Net/Common/SoftelControl/PROPERTYGRID
 *
 ********************* SniipSoftel *****************/
