/********************* SniipSoftel *****************
 *
 *   $Author: Serg_ch $
 *   $Revision: 6 $
 *   $Date: 24.01.07 20:16 $
 *
 ********************* SniipSoftel *****************/
using System;
using System.Collections;
using System.ComponentModel;
using System.Resources;
using System.Threading;

namespace Softel.Windows.Forms
{

	#region LocalizedPropertyDescriptor

	/// <summary>
	/// LocalizedPropertyDescriptor enhances the base class bay obtaining the display name for a property
	/// from the resource.
	/// </summary>
	public class LocalizedPropertyDescriptor : PropertyDescriptor
	{
		private PropertyDescriptor m_pdBase;
		private String m_LocalizedName = "";
		private String m_strLocalizedDescription = "";
		private ResourceManager m_rmDesc;
		private string m_strDisplayName = "";
		private string m_strDescription = "";
		private bool m_bVisible = true;
		private bool m_bReadOnly = false;
		private ArrayList m_arrList = null;

		public LocalizedPropertyDescriptor(PropertyDescriptor pdBase) : base(pdBase)
		{
			m_pdBase = pdBase;
			string tableName = "";
			int iCount = m_pdBase.Attributes.Count;
			int iIndex = 0;
			while(iIndex < iCount)
			{
				Attribute oAttrib = m_pdBase.Attributes[iIndex++];
				if (oAttrib.GetType().Equals(typeof (LocalizedPropertyAttribute)))
				{
					m_strDisplayName = ((LocalizedPropertyAttribute) oAttrib).Name;
					m_strDescription = ((LocalizedPropertyAttribute) oAttrib).Description;
					tableName = ((LocalizedPropertyAttribute) oAttrib).Table;
				}
			}
			// If no resource table is specified at the property level, then look if it is done
			// on the class level
			if (tableName.Length == 0)
			{
				LocalizedObjectAttribute[] attrs = (LocalizedObjectAttribute[]) m_pdBase.ComponentType.GetCustomAttributes(typeof (LocalizedObjectAttribute), false);
				if (attrs.Length > 0)
				{
					tableName = ((LocalizedObjectAttribute) attrs[0]).Table;
				}
			}
			// If still no resource table specified by attribute, then build it itself by using namespace and class name.
			if (tableName.Length == 0)
				tableName = m_pdBase.ComponentType.Namespace + "." + m_pdBase.ComponentType.Name;

			// If no display name id is specified by attribute, then construct it by using default display name (usually the property name) 
			if (m_strDisplayName.Length == 0)
				m_strDisplayName = this.m_pdBase.DisplayName;

			if (m_strDescription.Length == 0)
				m_strDescription = this.m_pdBase.Description;

			// Now use table name and display name id to access the resources.  
			m_rmDesc = new ResourceManager(tableName, m_pdBase.ComponentType.Assembly);
		}

		public override AttributeCollection Attributes
		{
			get
			{
				AttributeCollection attrs = m_pdBase.Attributes;
				int iCount = attrs.Count;
				int iIndex = 0;
				while(iIndex < iCount)
				{
					Attribute attr = attrs[iIndex++];
					if (attr is LocalizedCategoryAttribute)
					{
						LocalizedCategoryAttribute attrCat = (LocalizedCategoryAttribute) attr;
						attrCat.Assembly = m_pdBase.ComponentType.Assembly;
						attrCat.TableName = m_rmDesc.BaseName;
//						attrCat.TableName = m_pdBase.ComponentType.Namespace + "." + m_pdBase.ComponentType.Name;
					}
				}
				return attrs;
			}
		}

		public override bool CanResetValue(object component)
		{
			return m_pdBase.CanResetValue(component);
		}

		public override Type ComponentType
		{
			get { return m_pdBase.ComponentType; }
		}

		public override string DisplayName
		{
			get
			{
				string s = m_rmDesc.GetString(m_strDisplayName, Thread.CurrentThread.CurrentCulture);
				this.m_LocalizedName = (s != null) ? s : this.m_pdBase.DisplayName;

				return this.m_LocalizedName;
			}
		}

		public override string Description
		{
			get
			{
				string s = m_rmDesc.GetString(m_strDescription, Thread.CurrentThread.CurrentCulture);
				this.m_strLocalizedDescription = (s != null) ? s : this.m_pdBase.Description;

				return this.m_strLocalizedDescription;
			}
		}

		public override object GetValue(object component)
		{
			return this.m_pdBase.GetValue(component);
		}

		public override bool IsReadOnly
		{
			get
			{
//				return this.m_pdBase.IsReadOnly; 
				return m_bReadOnly;
			}
		}

		public bool ReadOnly
		{
			get { return m_bReadOnly; }
			set { m_bReadOnly = value; }
		}

		public override string Name
		{
			get { return this.m_pdBase.Name; }
		}

		public override Type PropertyType
		{
			get { return this.m_pdBase.PropertyType; }
		}

		public override void ResetValue(object component)
		{
			this.m_pdBase.ResetValue(component);
		}

		public override bool ShouldSerializeValue(object component)
		{
			return this.m_pdBase.ShouldSerializeValue(component);
		}

		public override void SetValue(object component, object value)
		{
			this.m_pdBase.SetValue(component, value);
		}

		public bool Visible
		{
			get { return m_bVisible; }
			set { m_bVisible = value; }
		}

		public ArrayList List
		{
			get { return m_arrList; }
			set { m_arrList = value; }
		}
	}

	#endregion

	#region LocalizedObject

	/// <summary>
	/// LocalizedObject implements ICustomTypeDescriptor to enable 
	/// required functionality to describe a type (class).<br></br>
	/// The main task of this class is to instantiate our own property descriptor 
	/// of type LocalizedPropertyDescriptor.  
	/// </summary>
//	[Serializable]
	public class LocalizedObject : ICustomTypeDescriptor
	{
		private PropertyDescriptorCollection LocalizedProps;

		public LocalizedObject()
		{
			LocalizedProps = null;
			GetProperties();
		}

		public String GetClassName()
		{
			return TypeDescriptor.GetClassName(this, true);
		}

		public AttributeCollection GetAttributes()
		{
			return TypeDescriptor.GetAttributes(this, true);
		}

		public String GetComponentName()
		{
			return TypeDescriptor.GetComponentName(this, true);
		}

		public TypeConverter GetConverter()
		{
			return TypeDescriptor.GetConverter(this, true);
		}

		public EventDescriptor GetDefaultEvent()
		{
			return TypeDescriptor.GetDefaultEvent(this, true);
		}

		public PropertyDescriptor GetDefaultProperty()
		{
			return TypeDescriptor.GetDefaultProperty(this, true);
		}

		public object GetEditor(Type editorBaseType)
		{
			return TypeDescriptor.GetEditor(this, editorBaseType, true);
		}

		public EventDescriptorCollection GetEvents(Attribute[] attributes)
		{
			return TypeDescriptor.GetEvents(this, attributes, true);
		}

		public EventDescriptorCollection GetEvents()
		{
			return TypeDescriptor.GetEvents(this, true);
		}

		/// <summary>
		/// Called to get the properties of a type.
		/// </summary>
		/// <param name="attributes"></param>
		/// <returns></returns>
		public PropertyDescriptorCollection GetProperties(Attribute[] attributes)
		{
			PropertyDescriptorCollection pdcProps = null;
			PropertyDescriptorCollection pdcAttributes = TypeDescriptor.GetProperties(this, attributes, true);
			if (LocalizedProps != null)
			{
				// Get the collection of properties
				pdcProps = new PropertyDescriptorCollection(null);
				int iCount = LocalizedProps.Count;
				int iIndex = 0;
				while(iIndex < iCount)
				{
					PropertyDescriptor pdProp = LocalizedProps[iIndex++];
					LocalizedPropertyDescriptor gProp = (LocalizedPropertyDescriptor) pdProp;
					if (gProp.Visible)
					{
						foreach (PropertyDescriptor pdAtt in pdcAttributes)
						{
							if (pdProp.Name == pdAtt.Name)
							{
								pdcProps.Add(pdProp);
								break;
							}
						}
					}
				}
			}
			return pdcProps;
		}

		public PropertyDescriptorCollection GetAllProperties()
		{
			PropertyDescriptorCollection baseProps = TypeDescriptor.GetProperties(this, true);
			PropertyDescriptorCollection newProps = new PropertyDescriptorCollection(null);
			int iCount = baseProps.Count;
			int iIndex = 0;
			while(iIndex < iCount)
			{
				PropertyDescriptor oProp = baseProps[iIndex++];
				newProps.Add(new LocalizedPropertyDescriptor(oProp));
			}
			return newProps;
		}

		public void Refresh()
		{
			LocalizedProps = null;
			GetProperties();
		}

		public PropertyDescriptorCollection GetProperties()
		{
			// Only do once
			if (LocalizedProps == null)
			{
				// Get the collection of properties
				PropertyDescriptorCollection baseProps = TypeDescriptor.GetProperties(this, true);
				LocalizedProps = new PropertyDescriptorCollection(null);

				// For each property use a property descriptor of our own that is able to be Localized
				int iCount = baseProps.Count;
				int iIndex = 0;
				while(iIndex < iCount)
				{
					PropertyDescriptor oProp = baseProps[iIndex++];
					LocalizedProps.Add(new LocalizedPropertyDescriptor(oProp));
				}
			}
			return LocalizedProps;
		}

		public object GetPropertyOwner(PropertyDescriptor pd)
		{
			return this;
		}

		public PropertyDescriptor GetPropertyDescriptor(string strDisplayName)
		{
			PropertyDescriptor pdReturn = null;
			if (LocalizedProps != null)
			{
				PropertyDescriptorCollection pdcBaseProps = GetProperties();
				int iCount = pdcBaseProps.Count;
				int iIndex = 0;
				while(iIndex < iCount)
				{
					PropertyDescriptor pdCurrent = pdcBaseProps[iIndex++];
					if (pdCurrent.Name == strDisplayName)
					{
						pdReturn = pdCurrent;
						break;
					}
				}
			}
			return pdReturn;
		}
	}

	#endregion
}
/********************* SniipSoftel *****************
 *
 *    $History: LocalizedDescriptor.cs $
 *   
 *   *****************  Version 6  *****************
 *   User: Serg_ch      Date: 24.01.07   Time: 20:16
 *   Updated in $/Net/Common/SoftelControl/PropertyGrid
 *   
 *   *****************  Version 5  *****************
 *   User: Autobuild    Date: 19.12.06   Time: 23:18
 *   Updated in $/Net/Common/SoftelControl/PROPERTYGRID
 *
 ********************* SniipSoftel *****************/
